@@ -3,6 +3,9 @@
 This file contains constant definitions.
 """
 
+import math as _math
+
+
 # MEASUREMENTS
 
 #:
@@ -20,6 +23,9 @@ MEASUREMENT = 'measurement'
 #:
 TIME = 'time'
 
+#: Time value that indicates steady-state measurements
+TIME_STEADY_STATE = _math.inf
+
 #:
 OBSERVABLE_PARAMETERS = 'observableParameters'
 
@@ -258,6 +264,8 @@ MEASUREMENT_FILES = 'measurement_files'
 OBSERVABLE_FILES = 'observable_files'
 #:
 VISUALIZATION_FILES = 'visualization_files'
+#:
+EXTENSIONS = 'extensions'
 
 
 # MORE
@@ -268,3 +276,6 @@ SIMULATION = 'simulation'
 RESIDUAL = 'residual'
 #:
 NOISE_VALUE = 'noiseValue'
+
+# separator for multiple parameter values (bounds, observableParameters, ...)
+PARAMETER_SEPARATOR = ';'
