@@ -1,11 +1,12 @@
 """Functions operating on the PEtab condition table"""
 
-from typing import Iterable, Optional, List, Union
+from pathlib import Path
+from typing import Iterable, List, Optional, Union
 
 import numpy as np
 import pandas as pd
 
-from . import lint, core
+from . import core, lint
 from .C import *
 
 __all__ = ['get_condition_df', 'write_condition_df', 'create_condition_df',
@@ -13,7 +14,7 @@ __all__ = ['get_condition_df', 'write_condition_df', 'create_condition_df',
 
 
 def get_condition_df(
-        condition_file: Union[str, pd.DataFrame, None]
+        condition_file: Union[str, pd.DataFrame, Path, None]
 ) -> pd.DataFrame:
     """Read the provided condition file into a ``pandas.Dataframe``
 
@@ -25,7 +26,7 @@ def get_condition_df(
     if condition_file is None:
         return condition_file
 
-    if isinstance(condition_file, str):
+    if isinstance(condition_file, (str, Path)):
         condition_file = pd.read_csv(condition_file, sep='\t',
                                      float_precision='round_trip')
 
@@ -44,13 +45,14 @@ def get_condition_df(
     return condition_file
 
 
-def write_condition_df(df: pd.DataFrame, filename: str) -> None:
+def write_condition_df(df: pd.DataFrame, filename: Union[str, Path]) -> None:
     """Write PEtab condition table
 
     Arguments:
         df: PEtab condition table
         filename: Destination file name
     """
+    df = get_condition_df(df)
     df.to_csv(filename, sep='\t', index=True)
 
 
@@ -91,9 +93,8 @@ def get_parametric_overrides(condition_df: pd.DataFrame) -> List[str]:
     Returns:
         List of parameter IDs that are mapped in a condition-specific way
     """
-    constant_parameters = list(
-        set(condition_df.columns.values.tolist()) - {CONDITION_ID,
-                                                     CONDITION_NAME})
+    constant_parameters = (set(condition_df.columns.values.tolist())
+                           - {CONDITION_ID, CONDITION_NAME})
     result = []
 
     for column in constant_parameters:
@@ -102,7 +103,5 @@ def get_parametric_overrides(condition_df: pd.DataFrame) -> List[str]:
 
         floatified = condition_df.loc[:, column].apply(core.to_float_if_float)
 
-        for x in floatified:
-            if not isinstance(x, float):
-                result.append(x)
+        result.extend(x for x in floatified if not isinstance(x, float))
     return result
