@@ -7,13 +7,13 @@ import re
 from typing import Optional, Iterable, Any
 from collections import Counter
 
-import libsbml
 import numpy as np
 import pandas as pd
 import sympy as sp
 
 import petab
-from . import (core, parameters, sbml, measurements)
+from . import (core, parameters, measurements)
+from .models import Model
 from .C import *  # noqa: F403
 
 logger = logging.getLogger(__name__)
@@ -60,9 +60,7 @@ def _check_df(df: pd.DataFrame, req_cols: Iterable, name: str) -> None:
     Raises:
           AssertionError: if a column is missing
     """
-    cols_set = df.columns.values
-    missing_cols = set(req_cols) - set(cols_set)
-    if missing_cols:
+    if missing_cols := set(req_cols) - set(df.columns.values):
         raise AssertionError(
             f"DataFrame {name} requires the columns {missing_cols}.")
 
@@ -85,12 +83,16 @@ def assert_no_leading_trailing_whitespace(
 
 
 def check_condition_df(
-        df: pd.DataFrame, sbml_model: Optional[libsbml.Model] = None) -> None:
+        df: pd.DataFrame,
+        model: Optional[Model] = None,
+        observable_df: Optional[pd.DataFrame] = None
+) -> None:
     """Run sanity checks on PEtab condition table
 
     Arguments:
         df: PEtab condition DataFrame
-        sbml_model: SBML Model for additional checking of parameter IDs
+        model: Model for additional checking of parameter IDs
+        observable_df: PEtab observables DataFrame
 
     Raises:
         AssertionError: in case of problems
@@ -101,28 +103,33 @@ def check_condition_df(
     _check_df(df, req_cols, "condition")
 
     # Check for correct index
-    if not df.index.name == CONDITION_ID:
+    if df.index.name != CONDITION_ID:
         raise AssertionError(
             f"Condition table has wrong index {df.index.name}."
             f"expected {CONDITION_ID}.")
 
     check_ids(df.index.values, kind='condition')
 
+    if not df.index.is_unique:
+        raise AssertionError("Non-unique condition IDs: "
+                             f"{df.index.values[df.index.duplicated()]}")
+
     for column_name in req_cols:
         if not np.issubdtype(df[column_name].dtype, np.number):
             assert_no_leading_trailing_whitespace(
                 df[column_name].values, column_name)
 
-    if sbml_model is not None:
+    if model is not None:
+        allowed_cols = set(model.get_valid_ids_for_condition_table())
+        if observable_df is not None:
+            allowed_cols |= set(petab.get_output_parameters(
+                model=model, observable_df=observable_df))
         for column_name in df.columns:
             if column_name != CONDITION_NAME \
-                    and sbml_model.getParameter(column_name) is None \
-                    and sbml_model.getSpecies(column_name) is None \
-                    and sbml_model.getCompartment(column_name) is None:
+                    and column_name not in allowed_cols:
                 raise AssertionError(
                     "Condition table contains column for unknown entity '"
-                    f"{column_name}'. Column names must match parameter, "
-                    "species or compartment IDs specified in the SBML model.")
+                    f"{column_name}'.")
 
 
 def check_measurement_df(df: pd.DataFrame,
@@ -152,14 +159,9 @@ def check_measurement_df(df: pd.DataFrame,
                 df[column_name].values, column_name)
 
     if observable_df is not None:
-        # Check all observables are defined
-        observables_defined = set(observable_df.index.values)
-        observables_used = set(df[OBSERVABLE_ID])
-        observables_undefined = observables_used - observables_defined
-        if observables_undefined:
-            raise ValueError(f"Observables {observables_undefined} used in "
-                             "measurement table but not defined in "
-                             "observables table.")
+        assert_measured_observables_defined(df, observable_df)
+        measurements.assert_overrides_match_parameter_count(
+            df, observable_df)
 
         if OBSERVABLE_TRANSFORMATION in observable_df:
             # Check for positivity of measurements in case of
@@ -174,18 +176,13 @@ def check_measurement_df(df: pd.DataFrame,
                                      f'transformation {trafo} must be '
                                      f'positive, but {measurement} <= 0.')
 
-    if observable_df is not None:
-        assert_measured_observables_defined(df, observable_df)
-        measurements.assert_overrides_match_parameter_count(
-            df, observable_df)
-
     assert_measurements_not_null(df)
     assert_measurements_numeric(df)
 
 
 def check_parameter_df(
         df: pd.DataFrame,
-        sbml_model: Optional[libsbml.Model] = None,
+        model: Optional[Model] = None,
         observable_df: Optional[pd.DataFrame] = None,
         measurement_df: Optional[pd.DataFrame] = None,
         condition_df: Optional[pd.DataFrame] = None) -> None:
@@ -193,7 +190,7 @@ def check_parameter_df(
 
     Arguments:
         df: PEtab condition DataFrame
-        sbml_model: SBML Model for additional checking of parameter IDs
+        model: Model for additional checking of parameter IDs
         observable_df: PEtab observable table for additional checks
         measurement_df: PEtab measurement table for additional checks
         condition_df: PEtab condition table for additional checks
@@ -204,7 +201,7 @@ def check_parameter_df(
 
     _check_df(df, PARAMETER_DF_REQUIRED_COLS[1:], "parameter")
 
-    if not df.index.name == PARAMETER_ID:
+    if df.index.name != PARAMETER_ID:
         raise AssertionError(
             f"Parameter table has wrong index {df.index.name}."
             f"expected {PARAMETER_ID}.")
@@ -230,10 +227,11 @@ def check_parameter_df(
                                  f"but column {NOMINAL_VALUE} is missing.")
         try:
             df.loc[non_estimated_par_ids, NOMINAL_VALUE].apply(float)
-        except ValueError:
-            raise AssertionError("Expected numeric values for "
-                                 f"`{NOMINAL_VALUE}` in parameter table for "
-                                 "all non-estimated parameters.")
+        except ValueError as e:
+            raise AssertionError(
+                f"Expected numeric values for `{NOMINAL_VALUE}` in parameter "
+                "table for all non-estimated parameters."
+            ) from e
 
     assert_parameter_id_is_string(df)
     assert_parameter_scale_is_valid(df)
@@ -243,10 +241,10 @@ def check_parameter_df(
     check_parameter_bounds(df)
     assert_parameter_prior_type_is_valid(df)
 
-    if sbml_model and measurement_df is not None \
+    if model and measurement_df is not None \
             and condition_df is not None:
         assert_all_parameters_present_in_parameter_df(
-            df, sbml_model, observable_df, measurement_df, condition_df)
+            df, model, observable_df, measurement_df, condition_df)
 
 
 def check_observable_df(observable_df: pd.DataFrame) -> None:
@@ -283,21 +281,28 @@ def check_observable_df(observable_df: pd.DataFrame) -> None:
         try:
             sp.sympify(obs)
         except sp.SympifyError as e:
-            raise AssertionError(f"Cannot parse expression '{obs}' "
-                                 f"for observable {row.Index}: {e}")
+            raise AssertionError(
+                f"Cannot parse expression '{obs}' "
+                f"for observable {row.Index}: {e}") from e
 
         noise = getattr(row, NOISE_FORMULA)
         try:
-            sp.sympify(noise)
+            sympified_noise = sp.sympify(noise)
+            if sympified_noise is None \
+                    or (sympified_noise.is_Number
+                        and not sympified_noise.is_finite):
+                raise AssertionError(f"No or non-finite {NOISE_FORMULA} "
+                                     f"given for observable {row.Index}.")
         except sp.SympifyError as e:
-            raise AssertionError(f"Cannot parse expression '{noise}' "
-                                 f"for noise model for observable "
-                                 f"{row.Index}: {e}")
+            raise AssertionError(
+                f"Cannot parse expression '{noise}' "
+                f"for noise model for observable " f"{row.Index}: {e}"
+            ) from e
 
 
 def assert_all_parameters_present_in_parameter_df(
         parameter_df: pd.DataFrame,
-        sbml_model: libsbml.Model,
+        model: Model,
         observable_df: pd.DataFrame,
         measurement_df: pd.DataFrame,
         condition_df: pd.DataFrame) -> None:
@@ -306,7 +311,7 @@ def assert_all_parameters_present_in_parameter_df(
 
     Arguments:
         parameter_df: PEtab parameter DataFrame
-        sbml_model: PEtab SBML Model
+        model: model
         observable_df: PEtab observable table
         measurement_df: PEtab measurement table
         condition_df: PEtab condition table
@@ -316,11 +321,11 @@ def assert_all_parameters_present_in_parameter_df(
     """
 
     required = parameters.get_required_parameters_for_parameter_table(
-        sbml_model=sbml_model, condition_df=condition_df,
+        model=model, condition_df=condition_df,
         observable_df=observable_df, measurement_df=measurement_df)
 
     allowed = parameters.get_valid_parameters_for_parameter_table(
-        sbml_model=sbml_model, condition_df=condition_df,
+        model=model, condition_df=condition_df,
         observable_df=observable_df, measurement_df=measurement_df)
 
     actual = set(parameter_df.index)
@@ -339,7 +344,8 @@ def assert_all_parameters_present_in_parameter_df(
 
 def assert_measured_observables_defined(
         measurement_df: pd.DataFrame,
-        observable_df: pd.DataFrame) -> None:
+        observable_df: pd.DataFrame
+) -> None:
     """Check if all observables in the measurement table have been defined in
     the observable table
 
@@ -353,12 +359,11 @@ def assert_measured_observables_defined(
 
     used_observables = set(measurement_df[OBSERVABLE_ID].values)
     defined_observables = set(observable_df.index.values)
-    undefined_observables = used_observables - defined_observables
-
-    if undefined_observables:
+    if undefined_observables := (used_observables - defined_observables):
         raise AssertionError(
-            "Undefined observables in measurement file: "
-            f"{undefined_observables}.")
+            f"Observables {undefined_observables} used in "
+            "measurement table but not defined in observables table."
+        )
 
 
 def condition_table_is_parameter_free(condition_df: pd.DataFrame) -> bool:
@@ -466,7 +471,7 @@ def check_parameter_bounds(parameter_df: pd.DataFrame) -> None:
                 raise AssertionError(
                     f"{LOWER_BOUND} greater than {UPPER_BOUND} for "
                     f"{PARAMETER_ID} {row.name}.")
-            if (row[LOWER_BOUND] <= 0.0 or row[UPPER_BOUND] < 0.0) \
+            if (row[LOWER_BOUND] < 0.0 or row[UPPER_BOUND] < 0.0) \
                     and row[PARAMETER_SCALE] in [LOG, LOG10]:
                 raise AssertionError(
                     f"Bounds for {row[PARAMETER_SCALE]} scaled parameter "
@@ -530,16 +535,19 @@ def assert_parameter_prior_parameters_are_valid(
                 continue
             # parse parameters
             try:
-                pars = tuple([float(val) for val in pars_str.split(';')])
-            except ValueError:
+                pars = tuple(
+                    float(val) for val in pars_str.split(PARAMETER_SEPARATOR)
+                )
+            except ValueError as e:
                 raise AssertionError(
-                    f"Could not parse prior parameters '{pars_str}'.")
+                    f"Could not parse prior parameters '{pars_str}'.") from e
+
             # all distributions take 2 parameters
             if len(pars) != 2:
                 raise AssertionError(
                     f"The prior parameters '{pars}' do not contain the "
-                    "expected number of entries (currently 'par1;par2' "
-                    "for all prior types).")
+                    "expected number of entries (currently 'par1"
+                    f"{PARAMETER_SEPARATOR}par2' for all prior types).")
 
 
 def assert_parameter_estimate_is_boolean(parameter_df: pd.DataFrame) -> None:
@@ -754,14 +762,19 @@ def lint_problem(problem: 'petab.Problem') -> bool:
     # pylint: disable=too-many-statements
     errors_occurred = False
 
+    if problem.extensions_config:
+        logger.warning(
+            "Validation of PEtab extensions is not yet implemented, "
+            "but the given problem uses the following extensions: "
+            f"{'', ''.join(problem.extensions_config.keys())}"
+        )
+
     # Run checks on individual files
-    if problem.sbml_model is not None:
-        logger.info("Checking SBML model...")
-        errors_occurred |= not sbml.is_sbml_consistent(
-            problem.sbml_model.getSBMLDocument())
-        sbml.log_sbml_errors(problem.sbml_model.getSBMLDocument())
+    if problem.model is not None:
+        logger.info("Checking model...")
+        errors_occurred |= not problem.model.is_valid()
     else:
-        logger.warning("SBML model not available. Skipping.")
+        logger.warning("Model not available. Skipping.")
 
     if problem.measurement_df is not None:
         logger.info("Checking measurement table...")
@@ -781,7 +794,8 @@ def lint_problem(problem: 'petab.Problem') -> bool:
     if problem.condition_df is not None:
         logger.info("Checking condition table...")
         try:
-            check_condition_df(problem.condition_df, problem.sbml_model)
+            check_condition_df(problem.condition_df, problem.model,
+                               problem.observable_df)
         except AssertionError as e:
             logger.error(e)
             errors_occurred = True
@@ -795,9 +809,9 @@ def lint_problem(problem: 'petab.Problem') -> bool:
         except AssertionError as e:
             logger.error(e)
             errors_occurred = True
-        if problem.sbml_model is not None:
+        if problem.model is not None:
             for obs_id in problem.observable_df.index:
-                if problem.sbml_model.getElementBySId(obs_id):
+                if problem.model.has_entity_with_id(obs_id):
                     logger.error(f"Observable ID {obs_id} shadows model "
                                  "entity.")
                     errors_occurred = True
@@ -807,7 +821,7 @@ def lint_problem(problem: 'petab.Problem') -> bool:
     if problem.parameter_df is not None:
         logger.info("Checking parameter table...")
         try:
-            check_parameter_df(problem.parameter_df, problem.sbml_model,
+            check_parameter_df(problem.parameter_df, problem.model,
                                problem.observable_df,
                                problem.measurement_df, problem.condition_df)
         except AssertionError as e:
@@ -816,11 +830,11 @@ def lint_problem(problem: 'petab.Problem') -> bool:
     else:
         logger.warning("Parameter table not available. Skipping.")
 
-    if problem.sbml_model is not None and problem.condition_df is not None \
+    if problem.model is not None and problem.condition_df is not None \
             and problem.parameter_df is not None:
         try:
             assert_model_parameters_in_condition_or_parameter_table(
-                problem.sbml_model,
+                problem.model,
                 problem.condition_df,
                 problem.parameter_df
             )
@@ -831,7 +845,7 @@ def lint_problem(problem: 'petab.Problem') -> bool:
     if errors_occurred:
         logger.error('Not OK')
     elif problem.measurement_df is None or problem.condition_df is None \
-            or problem.sbml_model is None or problem.parameter_df is None \
+            or problem.model is None or problem.parameter_df is None \
             or problem.observable_df is None:
         logger.warning('Not all files of the PEtab problem definition could '
                        'be checked.')
@@ -842,46 +856,54 @@ def lint_problem(problem: 'petab.Problem') -> bool:
 
 
 def assert_model_parameters_in_condition_or_parameter_table(
-        sbml_model: libsbml.Model,
+        model: Model,
         condition_df: pd.DataFrame,
         parameter_df: pd.DataFrame) -> None:
-    """Model parameters that are targets of AssignmentRule must not be present
-    in parameter table or in condition table columns. Other parameters must
-    only be present in either in parameter table or condition table columns.
-    Check that.
+    """Model parameters that are rule targets must not be present in the
+    parameter table. Other parameters must only be present in either in
+    parameter table or condition table columns. Check that.
 
     Arguments:
         parameter_df: PEtab parameter DataFrame
-        sbml_model: PEtab SBML Model
+        model: PEtab model
         condition_df: PEtab condition table
 
     Raises:
         AssertionError: in case of problems
     """
+    allowed_in_condition_cols = set(model.get_valid_ids_for_condition_table())
+    allowed_in_parameter_table = \
+        set(model.get_valid_parameters_for_parameter_table())
+    entities_in_condition_table = set(condition_df.columns) - {CONDITION_NAME}
+    entities_in_parameter_table = set(parameter_df.index.values)
 
-    for parameter in sbml_model.getListOfParameters():
-        parameter_id = parameter.getId()
+    disallowed_in_condition = {
+        x for x in (entities_in_condition_table - allowed_in_condition_cols)
+        # we only check model entities here, not output parameters
+        if model.has_entity_with_id(x)
+    }
+    if disallowed_in_condition:
+        is_or_are = "is" if len(disallowed_in_condition) == 1 else "are"
+        raise AssertionError(f"{disallowed_in_condition} {is_or_are} not "
+                             "allowed to occur in condition table "
+                             "columns.")
 
-        if parameter_id.startswith('observableParameter'):
-            continue
-        if parameter_id.startswith('noiseParameter'):
-            continue
+    disallowed_in_parameters = {
+        x for x in (entities_in_parameter_table - allowed_in_parameter_table)
+        # we only check model entities here, not output parameters
+        if model.has_entity_with_id(x)
+    }
 
-        is_assignee = \
-            sbml_model.getAssignmentRuleByVariable(parameter_id) is not None
-        in_parameter_df = parameter_id in parameter_df.index
-        in_condition_df = parameter_id in condition_df.columns
+    if disallowed_in_parameters:
+        is_or_are = "is" if len(disallowed_in_parameters) == 1 else "are"
+        raise AssertionError(f"{disallowed_in_parameters} {is_or_are} not "
+                             "allowed to occur in the parameters table.")
 
-        if is_assignee and (in_parameter_df or in_condition_df):
-            raise AssertionError(f"Model parameter '{parameter_id}' is target "
-                                 "of AssignmentRule, and thus, must not be "
-                                 "present in condition table or in parameter "
-                                 "table.")
-
-        if in_parameter_df and in_condition_df:
-            raise AssertionError(f"Model parameter '{parameter_id}' present "
-                                 "in both condition table and parameter "
-                                 "table.")
+    in_both = entities_in_condition_table & entities_in_parameter_table
+    if in_both:
+        is_or_are = "is" if len(in_both) == 1 else "are"
+        raise AssertionError(f"{in_both} {is_or_are} present in both "
+                             "the condition table and the parameter table.")
 
 
 def assert_measurement_conditions_present_in_condition_table(
@@ -903,9 +925,7 @@ def assert_measurement_conditions_present_in_condition_table(
         used_conditions |= \
             set(measurement_df[PREEQUILIBRATION_CONDITION_ID].dropna().values)
     available_conditions = set(condition_df.index.values)
-    missing_conditions = used_conditions - available_conditions
-
-    if missing_conditions:
+    if missing_conditions := (used_conditions - available_conditions):
         raise AssertionError("Measurement table references conditions that "
                              "are not specified in the condition table: "
                              + str(missing_conditions))
