@@ -1,32 +1,32 @@
 """Functions operating on the PEtab measurement table"""
 # noqa: F405
 
-
 import itertools
+import math
 import numbers
-from typing import List, Union, Dict
-from warnings import warn
+from pathlib import Path
+from typing import Dict, List, Union
 
 import numpy as np
 import pandas as pd
 
-from . import (lint, core, observables)
+from . import (core, lint, observables)
 from .C import *  # noqa: F403
 
 __all__ = ['assert_overrides_match_parameter_count',
            'create_measurement_df',
            'get_measurement_df',
            'get_measurement_parameter_ids',
-           'get_noise_distributions',
            'get_rows_for_condition',
            'get_simulation_conditions',
            'measurements_have_replicates',
+           'measurement_is_at_steady_state',
            'split_parameter_replacement_list',
            'write_measurement_df']
 
 
 def get_measurement_df(
-        measurement_file: Union[None, str, pd.DataFrame]
+        measurement_file: Union[None, str, Path, pd.DataFrame]
 ) -> pd.DataFrame:
     """
     Read the provided measurement file into a ``pandas.Dataframe``.
@@ -40,7 +40,7 @@ def get_measurement_df(
     if measurement_file is None:
         return measurement_file
 
-    if isinstance(measurement_file, str):
+    if isinstance(measurement_file, (str, Path)):
         measurement_file = pd.read_csv(measurement_file, sep='\t',
                                        float_precision='round_trip')
 
@@ -50,64 +50,17 @@ def get_measurement_df(
     return measurement_file
 
 
-def write_measurement_df(df: pd.DataFrame, filename: str) -> None:
+def write_measurement_df(df: pd.DataFrame, filename: Union[str, Path]) -> None:
     """Write PEtab measurement table
 
     Arguments:
         df: PEtab measurement table
         filename: Destination file name
     """
+    df = get_measurement_df(df)
     df.to_csv(filename, sep='\t', index=False)
 
 
-def get_noise_distributions(measurement_df: pd.DataFrame) -> dict:
-    """
-    Returns dictionary of cost definitions per observable, if specified.
-
-    Looks through all parameters satisfying `sbml_parameter_is_cost` and
-    return as dictionary.
-
-    Parameters:
-        measurement_df: PEtab measurement table
-
-    Returns:
-        Dictionary with `observableId` => `cost definition`
-    """
-    warn("This function will be removed in future releases.",
-         DeprecationWarning)
-
-    lint.assert_noise_distributions_valid(measurement_df)
-
-    # read noise distributions from measurement file
-    grouping_cols = core.get_notnull_columns(
-        measurement_df, [OBSERVABLE_ID, OBSERVABLE_TRANSFORMATION,
-                         NOISE_DISTRIBUTION])
-
-    observables = measurement_df.fillna('').groupby(grouping_cols).size()\
-        .reset_index()
-    noise_distrs = {}
-    for _, row in observables.iterrows():
-        # prefix id to get observable id
-        id_ = 'observable_' + row.observableId
-
-        # extract observable transformation and noise distribution,
-        # use lin+normal as default if none provided
-        obs_trafo = row.observableTransformation \
-            if OBSERVABLE_TRANSFORMATION in row \
-            and row.observableTransformation \
-            else LIN
-        noise_distr = row.noiseDistribution \
-            if NOISE_DISTRIBUTION in row \
-            and row.noiseDistribution \
-            else NORMAL
-        # add to noise distributions
-        noise_distrs[id_] = {
-            OBSERVABLE_TRANSFORMATION: obs_trafo,
-            NOISE_DISTRIBUTION: noise_distr}
-
-    return noise_distrs
-
-
 def get_simulation_conditions(measurement_df: pd.DataFrame) -> pd.DataFrame:
     """
     Create a table of separate simulation conditions. A simulation condition
@@ -123,6 +76,8 @@ def get_simulation_conditions(measurement_df: pd.DataFrame) -> pd.DataFrame:
         Missing 'preequilibrationConditionId's will be set to '' (empty
         string).
     """
+    if measurement_df.empty:
+        return pd.DataFrame(data={SIMULATION_CONDITION_ID: []})
     # find columns to group by (i.e. if not all nans).
     # can be improved by checking for identical condition vectors
     grouping_cols = core.get_notnull_columns(
@@ -200,7 +155,7 @@ def get_measurement_parameter_ids(measurement_df: pd.DataFrame) -> List[str]:
 
 def split_parameter_replacement_list(
         list_string: Union[str, numbers.Number],
-        delim: str = ';') -> List[Union[str, numbers.Number]]:
+        delim: str = PARAMETER_SEPARATOR) -> List[Union[str, numbers.Number]]:
     """
     Split values in observableParameters and noiseParameters in measurement
     table.
@@ -247,7 +202,7 @@ def create_measurement_df() -> pd.DataFrame:
         Created DataFrame
     """
 
-    df = pd.DataFrame(data={
+    return pd.DataFrame(data={
         OBSERVABLE_ID: [],
         PREEQUILIBRATION_CONDITION_ID: [],
         SIMULATION_CONDITION_ID: [],
@@ -259,8 +214,6 @@ def create_measurement_df() -> pd.DataFrame:
         REPLICATE_ID: []
     })
 
-    return df
-
 
 def measurements_have_replicates(measurement_df: pd.DataFrame) -> bool:
     """Tests whether the measurements come with replicates
@@ -281,7 +234,8 @@ def measurements_have_replicates(measurement_df: pd.DataFrame) -> bool:
 
 def assert_overrides_match_parameter_count(
         measurement_df: pd.DataFrame,
-        observable_df: pd.DataFrame) -> None:
+        observable_df: pd.DataFrame
+) -> None:
     """Ensure that number of parameters in the observable definition matches
     the number of overrides in ``measurement_df``
 
@@ -289,7 +243,6 @@ def assert_overrides_match_parameter_count(
         measurement_df: PEtab measurement table
         observable_df: PEtab observable table
     """
-
     # sympify only once and save number of parameters
     observable_parameters_count = {
         obs_id: len(observables.get_formula_placeholders(
@@ -306,10 +259,11 @@ def assert_overrides_match_parameter_count(
         # check observable parameters
         try:
             expected = observable_parameters_count[row[OBSERVABLE_ID]]
-        except KeyError:
+        except KeyError as e:
             raise ValueError(
                 f"Observable {row[OBSERVABLE_ID]} used in measurement table "
-                f"is not defined.")
+                f"is not defined.") from e
+
         actual = len(split_parameter_replacement_list(
             row.get(OBSERVABLE_PARAMETERS, None)))
         # No overrides are also allowed
@@ -331,14 +285,27 @@ def assert_overrides_match_parameter_count(
             if len(replacements) != expected:
                 raise AssertionError(
                     f'Mismatch of noise parameter overrides in:\n{row}\n'
-                    f'Expected {expected} but got {actual}')
+                    f'Expected {expected} but got {len(replacements)}')
         except KeyError:
             # no overrides defined, but a numerical sigma can be provided
             # anyways
-            if not len(replacements) == 1 \
+            if len(replacements) != 1 \
                     or not isinstance(replacements[0], numbers.Number):
                 raise AssertionError(
                     f'No placeholders have been specified in the noise model '
                     f'for observable {row[OBSERVABLE_ID]}, but parameter ID '
                     'or multiple overrides were specified in the '
                     'noiseParameters column.')
+
+
+def measurement_is_at_steady_state(time: float) -> bool:
+    """Check whether a measurement is at steady state.
+
+    Arguments:
+        time:
+            The time.
+
+    Returns:
+        Whether the measurement is at steady state.
+    """
+    return math.isinf(time)
