@@ -1,27 +1,30 @@
 """Functions for working with the PEtab observables table"""
 
-from collections import OrderedDict
-from typing import Union, List
-
-import libsbml
-import pandas as pd
 import re
+from collections import OrderedDict
+from pathlib import Path
+from typing import List, Union, Literal
+
+import pandas as pd
 import sympy as sp
 
-from . import lint, core
+from . import core, lint
 from .C import *  # noqa: F403
+from .models import Model
 
-__all__ = ['create_observable_df',
-           'get_formula_placeholders',
-           'get_observable_df',
-           'get_output_parameters',
-           'get_placeholders',
-           'write_observable_df']
+__all__ = [
+    'create_observable_df',
+    'get_formula_placeholders',
+    'get_observable_df',
+    'get_output_parameters',
+    'get_placeholders',
+    'write_observable_df'
+]
 
 
 def get_observable_df(
-        observable_file: Union[str, pd.DataFrame, None]
-) -> pd.DataFrame:
+        observable_file: Union[str, pd.DataFrame, Path, None]
+) -> Union[pd.DataFrame, None]:
     """
     Read the provided observable file into a ``pandas.Dataframe``.
 
@@ -34,7 +37,7 @@ def get_observable_df(
     if observable_file is None:
         return observable_file
 
-    if isinstance(observable_file, str):
+    if isinstance(observable_file, (str, Path)):
         observable_file = pd.read_csv(observable_file, sep='\t',
                                       float_precision='round_trip')
 
@@ -53,30 +56,31 @@ def get_observable_df(
     return observable_file
 
 
-def write_observable_df(df: pd.DataFrame, filename: str) -> None:
+def write_observable_df(df: pd.DataFrame, filename: Union[str, Path]) -> None:
     """Write PEtab observable table
 
     Arguments:
         df: PEtab observable table
         filename: Destination file name
     """
+    df = get_observable_df(df)
     df.to_csv(filename, sep='\t', index=True)
 
 
 def get_output_parameters(
         observable_df: pd.DataFrame,
-        sbml_model: libsbml.Model,
+        model: Model,
         observables: bool = True,
         noise: bool = True,
 ) -> List[str]:
     """Get output parameters
 
     Returns IDs of parameters used in observable and noise formulas that are
-    not defined in the SBML model.
+    not defined in the model.
 
     Arguments:
         observable_df: PEtab observable table
-        sbml_model: SBML model
+        model: The underlying model
         observables: Include parameters from observableFormulas
         noise: Include parameters from noiseFormulas
 
@@ -95,14 +99,17 @@ def get_output_parameters(
                            key=lambda symbol: symbol.name)
         for free_sym in free_syms:
             sym = str(free_sym)
-            if sbml_model.getElementBySId(sym) is None and sym != 'time':
+            if not model.symbol_allowed_in_observable_formula(sym):
                 output_parameters[sym] = None
 
     return list(output_parameters.keys())
 
 
-def get_formula_placeholders(formula_string: str, observable_id: str,
-                             override_type: str) -> List[str]:
+def get_formula_placeholders(
+        formula_string: str,
+        observable_id: str,
+        override_type: Literal['observable', 'noise'],
+) -> List[str]:
     """
     Get placeholder variables in noise or observable definition for the
     given observable ID.
@@ -110,8 +117,8 @@ def get_formula_placeholders(formula_string: str, observable_id: str,
     Arguments:
         formula_string: observable formula
         observable_id: ID of current observable
-        override_type: 'observable' or 'noise', depending on whether `formula`
-            is for observable or for noise model
+        override_type: ``'observable'`` or ``'noise'``, depending on whether
+            ``formula`` is for observable or for noise model
 
     Returns:
         List of placeholder parameter IDs in the order expected in the
@@ -188,6 +195,4 @@ def create_observable_df() -> pd.DataFrame:
         Created DataFrame
     """
 
-    df = pd.DataFrame(data={col: [] for col in OBSERVABLE_DF_COLS})
-
-    return df
+    return pd.DataFrame(data={col: [] for col in OBSERVABLE_DF_COLS})
