@@ -5,16 +5,17 @@ import logging
 import numbers
 import os
 import re
-from typing import Tuple, Dict, Union, Any, List, Optional, Iterable
+import warnings
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, Literal
 
 import libsbml
 import numpy as np
 import pandas as pd
 
-from . import lint, measurements, sbml, core, observables, parameters
-from . import ENV_NUM_THREADS
+from . import ENV_NUM_THREADS, core, lint, measurements, observables, \
+    parameters
 from .C import *  # noqa: F403
-
+from .models import Model
 
 logger = logging.getLogger(__name__)
 __all__ = ['get_optimization_to_simulation_parameter_mapping',
@@ -51,10 +52,11 @@ def get_optimization_to_simulation_parameter_mapping(
         warn_unmapped: Optional[bool] = True,
         scaled_parameters: bool = False,
         fill_fixed_parameters: bool = True,
-        allow_timepoint_specific_numeric_noise_parameters: bool = False
+        allow_timepoint_specific_numeric_noise_parameters: bool = False,
+        model: Model = None,
 ) -> List[ParMappingDictQuadruple]:
     """
-    Create list of mapping dicts from PEtab-problem to SBML parameters.
+    Create list of mapping dicts from PEtab-problem to model parameters.
 
     Mapping can be performed in parallel. The number of threads is controlled
     by the environment variable with the name of
@@ -64,8 +66,9 @@ def get_optimization_to_simulation_parameter_mapping(
         condition_df, measurement_df, parameter_df, observable_df:
             The dataframes in the PEtab format.
         sbml_model:
-            The sbml model with observables and noise specified according to
-            the PEtab format.
+            The SBML model (deprecated)
+        model:
+            The model.
         simulation_conditions:
             Table of simulation conditions as created by
             ``petab.get_simulation_conditions``.
@@ -100,6 +103,16 @@ def get_optimization_to_simulation_parameter_mapping(
         If no preequilibration condition is defined, the respective dicts will
         be empty. ``NaN`` is used where no mapping exists.
     """
+    if sbml_model:
+        warnings.warn("Passing a model via the `sbml_model` argument is "
+                      "deprecated, use `model=petab.models.sbml_model."
+                      "SbmlModel(...)` instead.", DeprecationWarning,
+                      stacklevel=2)
+        from petab.models.sbml_model import SbmlModel
+        if model:
+            raise ValueError("Arguments `model` and `sbml_model` are "
+                             "mutually exclusive.")
+        model = SbmlModel(sbml_model=sbml_model)
 
     # Ensure inputs are okay
     _perform_mapping_checks(
@@ -111,12 +124,11 @@ def get_optimization_to_simulation_parameter_mapping(
         simulation_conditions = measurements.get_simulation_conditions(
             measurement_df)
 
-    simulation_parameters = sbml.get_model_parameters(sbml_model,
-                                                      with_values=True)
-    # Add output parameters that are not already defined in the SBML model
+    simulation_parameters = dict(model.get_free_parameter_ids_with_values())
+    # Add output parameters that are not already defined in the model
     if observable_df is not None:
         output_parameters = observables.get_output_parameters(
-            observable_df=observable_df, sbml_model=sbml_model)
+            observable_df=observable_df, model=model)
         for par_id in output_parameters:
             simulation_parameters[par_id] = np.nan
 
@@ -129,7 +141,7 @@ def get_optimization_to_simulation_parameter_mapping(
             _map_condition,
             _map_condition_arg_packer(
                 simulation_conditions, measurement_df, condition_df,
-                parameter_df, sbml_model, simulation_parameters, warn_unmapped,
+                parameter_df, model, simulation_parameters, warn_unmapped,
                 scaled_parameters, fill_fixed_parameters,
                 allow_timepoint_specific_numeric_noise_parameters))
         return list(mapping)
@@ -141,7 +153,7 @@ def get_optimization_to_simulation_parameter_mapping(
             _map_condition,
             _map_condition_arg_packer(
                 simulation_conditions, measurement_df, condition_df,
-                parameter_df, sbml_model, simulation_parameters, warn_unmapped,
+                parameter_df, model, simulation_parameters, warn_unmapped,
                 scaled_parameters, fill_fixed_parameters,
                 allow_timepoint_specific_numeric_noise_parameters))
     return list(mapping)
@@ -152,7 +164,7 @@ def _map_condition_arg_packer(
         measurement_df,
         condition_df,
         parameter_df,
-        sbml_model,
+        model,
         simulation_parameters,
         warn_unmapped,
         scaled_parameters,
@@ -161,10 +173,12 @@ def _map_condition_arg_packer(
 ):
     """Helper function to pack extra arguments for _map_condition"""
     for _, condition in simulation_conditions.iterrows():
-        yield(condition, measurement_df, condition_df, parameter_df,
-              sbml_model, simulation_parameters, warn_unmapped,
-              scaled_parameters, fill_fixed_parameters,
-              allow_timepoint_specific_numeric_noise_parameters)
+        yield (
+            condition, measurement_df, condition_df, parameter_df,
+            model, simulation_parameters, warn_unmapped, scaled_parameters,
+            fill_fixed_parameters,
+            allow_timepoint_specific_numeric_noise_parameters
+        )
 
 
 def _map_condition(packed_args):
@@ -174,13 +188,20 @@ def _map_condition(packed_args):
     :py:func:`get_optimization_to_simulation_parameter_mapping`.
     """
 
-    (condition, measurement_df, condition_df, parameter_df, sbml_model,
+    (condition, measurement_df, condition_df, parameter_df, model,
      simulation_parameters, warn_unmapped, scaled_parameters,
      fill_fixed_parameters,
      allow_timepoint_specific_numeric_noise_parameters) = packed_args
 
-    cur_measurement_df = measurements.get_rows_for_condition(
-        measurement_df, condition)
+    cur_measurement_df = None
+    # Get the condition specific measurements for the current condition, but
+    # only if relevant for parameter mapping
+    if (OBSERVABLE_PARAMETERS in measurement_df
+        and measurement_df[OBSERVABLE_PARAMETERS].notna().any()) \
+        or (NOISE_PARAMETERS in measurement_df
+            and measurement_df[NOISE_PARAMETERS].notna().any()):
+        cur_measurement_df = measurements.get_rows_for_condition(
+            measurement_df, condition)
 
     if PREEQUILIBRATION_CONDITION_ID not in condition \
             or not isinstance(condition[PREEQUILIBRATION_CONDITION_ID], str) \
@@ -192,7 +213,7 @@ def _map_condition(packed_args):
             condition_id=condition[PREEQUILIBRATION_CONDITION_ID],
             is_preeq=True,
             cur_measurement_df=cur_measurement_df,
-            sbml_model=sbml_model,
+            model=model,
             condition_df=condition_df,
             parameter_df=parameter_df,
             simulation_parameters=simulation_parameters,
@@ -207,7 +228,7 @@ def _map_condition(packed_args):
         condition_id=condition[SIMULATION_CONDITION_ID],
         is_preeq=False,
         cur_measurement_df=cur_measurement_df,
-        sbml_model=sbml_model,
+        model=model,
         condition_df=condition_df,
         parameter_df=parameter_df,
         simulation_parameters=simulation_parameters,
@@ -224,15 +245,16 @@ def _map_condition(packed_args):
 def get_parameter_mapping_for_condition(
         condition_id: str,
         is_preeq: bool,
-        cur_measurement_df: pd.DataFrame,
-        sbml_model: libsbml.Model,
-        condition_df: pd.DataFrame,
+        cur_measurement_df: Optional[pd.DataFrame],
+        sbml_model: libsbml.Model = None,
+        condition_df: pd.DataFrame = None,
         parameter_df: pd.DataFrame = None,
         simulation_parameters: Optional[Dict[str, str]] = None,
         warn_unmapped: bool = True,
         scaled_parameters: bool = False,
         fill_fixed_parameters: bool = True,
         allow_timepoint_specific_numeric_noise_parameters: bool = False,
+        model: Model = None,
 ) -> Tuple[ParMappingDict, ScaleMappingDict]:
     """
     Create dictionary of parameter value and parameter scale mappings from
@@ -244,14 +266,16 @@ def get_parameter_mapping_for_condition(
         is_preeq:
             If ``True``, output parameters will not be mapped
         cur_measurement_df:
-            Measurement sub-table for current condition
+            Measurement sub-table for current condition, can be ``None`` if
+            not relevant for parameter mapping
         condition_df:
             PEtab condition DataFrame
         parameter_df:
             PEtab parameter DataFrame
         sbml_model:
-            The sbml model with observables and noise specified according to
-            the PEtab format used to retrieve simulation parameter IDs.
+            The SBML model (deprecated)
+        model:
+            The model.
         simulation_parameters:
             Model simulation parameter IDs mapped to parameter values (output
             of ``petab.sbml.get_model_parameters(.., with_values=True)``).
@@ -278,17 +302,29 @@ def get_parameter_mapping_for_condition(
         Second dictionary mapping model parameter IDs to their scale.
         ``NaN`` is used where no mapping exists.
     """
-    _perform_mapping_checks(
-        cur_measurement_df,
-        allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
-        allow_timepoint_specific_numeric_noise_parameters)
+    if sbml_model:
+        warnings.warn("Passing a model via the `sbml_model` argument is "
+                      "deprecated, use `model=petab.models.sbml_model."
+                      "SbmlModel(...)` instead.", DeprecationWarning,
+                      stacklevel=2)
+        from petab.models.sbml_model import SbmlModel
+        if model:
+            raise ValueError("Arguments `model` and `sbml_model` are "
+                             "mutually exclusive.")
+        model = SbmlModel(sbml_model=sbml_model)
+
+    if cur_measurement_df is not None:
+        _perform_mapping_checks(
+            cur_measurement_df,
+            allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
+            allow_timepoint_specific_numeric_noise_parameters)
 
     if simulation_parameters is None:
-        simulation_parameters = sbml.get_model_parameters(sbml_model,
-                                                          with_values=True)
+        simulation_parameters = dict(
+            model.get_free_parameter_ids_with_values())
 
     # NOTE: order matters here - the former is overwritten by the latter:
-    #  SBML model < condition table < measurement < table parameter table
+    #  model < condition table < measurement < table parameter table
 
     # initialize mapping dicts
     # for the case of matching simulation and optimization parameter vector
@@ -298,13 +334,14 @@ def get_parameter_mapping_for_condition(
 
     # not strictly necessary for preequilibration, be we do it to have
     # same length of parameter vectors
-    _apply_output_parameter_overrides(par_mapping, cur_measurement_df)
+    if cur_measurement_df is not None:
+        _apply_output_parameter_overrides(par_mapping, cur_measurement_df)
 
     if not is_preeq:
         handle_missing_overrides(par_mapping, warn=warn_unmapped)
 
     _apply_condition_parameters(par_mapping, scale_mapping, condition_id,
-                                condition_df, sbml_model)
+                                condition_df, model)
     _apply_parameter_table(par_mapping, scale_mapping,
                            parameter_df, scaled_parameters,
                            fill_fixed_parameters)
@@ -327,7 +364,8 @@ def _output_parameters_to_nan(mapping: ParMappingDict) -> None:
 
 def _apply_output_parameter_overrides(
         mapping: ParMappingDict,
-        cur_measurement_df: pd.DataFrame) -> None:
+        cur_measurement_df: pd.DataFrame
+) -> None:
     """
     Apply output parameter overrides to the parameter mapping dict for a given
     condition as defined in the measurement table (``observableParameter``,
@@ -355,8 +393,9 @@ def _apply_output_parameter_overrides(
 def _apply_overrides_for_observable(
         mapping: ParMappingDict,
         observable_id: str,
-        override_type: str,
-        overrides: List[str]) -> None:
+        override_type: Literal['observable', 'noise'],
+        overrides: List[str],
+) -> None:
     """
     Apply parameter-overrides for observables and noises to mapping
     matrix.
@@ -364,7 +403,7 @@ def _apply_overrides_for_observable(
     Arguments:
         mapping: mapping dict to which to apply overrides
         observable_id: observable ID
-        override_type: 'observable' or 'noise'
+        override_type: ``'observable'`` or ``'noise'``
         overrides: list of overrides for noise or observable parameters
     """
     for i, override in enumerate(overrides):
@@ -372,11 +411,13 @@ def _apply_overrides_for_observable(
         mapping[overridee_id] = override
 
 
-def _apply_condition_parameters(par_mapping: ParMappingDict,
-                                scale_mapping: ScaleMappingDict,
-                                condition_id: str,
-                                condition_df: pd.DataFrame,
-                                sbml_model: libsbml.Model) -> None:
+def _apply_condition_parameters(
+        par_mapping: ParMappingDict,
+        scale_mapping: ScaleMappingDict,
+        condition_id: str,
+        condition_df: pd.DataFrame,
+        model: Model,
+) -> None:
     """Replace parameter IDs in parameter mapping dictionary by condition
     table parameter values (in-place).
 
@@ -389,23 +430,36 @@ def _apply_condition_parameters(par_mapping: ParMappingDict,
         if overridee_id == CONDITION_NAME:
             continue
 
-        # Species and compartments are handled elsewhere
-        if sbml_model.getSpecies(overridee_id) is not None:
-            continue
-        if sbml_model.getCompartment(overridee_id) is not None:
+        # Species, compartments, and rule targets are handled elsewhere
+        if model.is_state_variable(overridee_id):
             continue
 
         par_mapping[overridee_id] = core.to_float_if_float(
             condition_df.loc[condition_id, overridee_id])
+
+        if isinstance(par_mapping[overridee_id], numbers.Number) \
+                and np.isnan(par_mapping[overridee_id]):
+            # NaN in the condition table for an entity without time derivative
+            #  indicates that the model value should be used
+            try:
+                par_mapping[overridee_id] = \
+                    model.get_parameter_value(overridee_id)
+            except ValueError as e:
+                raise NotImplementedError(
+                    "Not sure how to handle NaN in condition table for "
+                    f"{overridee_id}."
+                ) from e
+
         scale_mapping[overridee_id] = LIN
 
 
-def _apply_parameter_table(par_mapping: ParMappingDict,
-                           scale_mapping: ScaleMappingDict,
-                           parameter_df: Optional[pd.DataFrame] = None,
-                           scaled_parameters: bool = False,
-                           fill_fixed_parameters: bool = True,
-                           ) -> None:
+def _apply_parameter_table(
+        par_mapping: ParMappingDict,
+        scale_mapping: ScaleMappingDict,
+        parameter_df: Optional[pd.DataFrame] = None,
+        scaled_parameters: bool = False,
+        fill_fixed_parameters: bool = True,
+) -> None:
     """Replace parameters from parameter table in mapping list for a given
     condition and set the corresponding scale.
 
@@ -489,15 +543,17 @@ def _perform_mapping_checks(
             "Timepoint-specific parameter overrides currently unsupported.")
 
 
-def handle_missing_overrides(mapping_par_opt_to_par_sim: ParMappingDict,
-                             warn: bool = True,
-                             condition_id: str = None) -> None:
+def handle_missing_overrides(
+        mapping_par_opt_to_par_sim: ParMappingDict,
+        warn: bool = True,
+        condition_id: str = None,
+) -> None:
     """
     Find all observable parameters and noise parameters that were not mapped
     and set their mapping to np.nan.
 
-    Assumes that parameters matching "(noise|observable)Parameter[0-9]+_" were
-    all supposed to be overwritten.
+    Assumes that parameters matching the regular expression
+    ``(noise|observable)Parameter[0-9]+_`` were all supposed to be overwritten.
 
     Parameters:
         mapping_par_opt_to_par_sim:
@@ -531,14 +587,15 @@ def merge_preeq_and_sim_pars_condition(
         condition_map_sim: ParMappingDict,
         condition_scale_map_preeq: ScaleMappingDict,
         condition_scale_map_sim: ScaleMappingDict,
-        condition: Any) -> None:
+        condition: Any,
+) -> None:
     """Merge preequilibration and simulation parameters and scales for a single
     condition while checking for compatibility.
 
     This function is meant for the case where we cannot have different
     parameters (and scales) for preequilibration and simulation. Therefore,
     merge both and ensure matching scales and parameters.
-    ``condition_map_sim`` and ``condition_scale_map_sim`` will ne modified in
+    ``condition_map_sim`` and ``condition_scale_map_sim`` will be modified in
     place.
 
     Arguments:
