@@ -1,22 +1,26 @@
 """Functions operating on the PEtab parameter table"""
 
 import numbers
-import pandas as pd
-import numpy as np
+import warnings
 from collections import OrderedDict
-from typing import Iterable, Set, List, Tuple, Dict, Union
+from pathlib import Path
+from typing import (
+    Dict, Iterable, List, Set, Tuple, Union, Optional, Literal, Sequence
+)
 
 import libsbml
+import numpy as np
+import pandas as pd
 
-from . import lint, core, measurements, conditions, observables
+from . import conditions, core, lint, measurements, observables
 from .C import *  # noqa: F403
+from .models import Model
 
 __all__ = ['create_parameter_df',
            'get_optimization_parameter_scaling',
            'get_optimization_parameters',
            'get_parameter_df',
            'get_priors_from_df',
-           'get_required_parameters_for_parameter_table',
            'get_valid_parameters_for_parameter_table',
            'map_scale',
            'map_unscale',
@@ -25,47 +29,41 @@ __all__ = ['create_parameter_df',
            'unscale',
            'write_parameter_df']
 
+PARAMETER_SCALE_ARGS = Literal['', 'lin', 'log', 'log10']
+
 
 def get_parameter_df(
-        parameter_file: Union[str, List[str], pd.DataFrame, None]
-) -> pd.DataFrame:
+        parameter_file: Union[str, Path, pd.DataFrame,
+                              Iterable[Union[str, Path, pd.DataFrame]], None]
+) -> Union[pd.DataFrame, None]:
     """
     Read the provided parameter file into a ``pandas.Dataframe``.
 
     Arguments:
-        parameter_file: Name of the file to read from or pandas.Dataframe.
+        parameter_file: Name of the file to read from or pandas.Dataframe,
+        or an Iterable.
 
     Returns:
-        Parameter DataFrame
+        Parameter ``DataFrame``, or ``None`` if ``None`` was passed.
     """
     if parameter_file is None:
-        return parameter_file
-
-    parameter_df = None
-
+        return None
     if isinstance(parameter_file, pd.DataFrame):
         parameter_df = parameter_file
-
-    if isinstance(parameter_file, str):
+    elif isinstance(parameter_file, (str, Path)):
         parameter_df = pd.read_csv(parameter_file, sep='\t',
                                    float_precision='round_trip')
+    elif isinstance(parameter_file, Iterable):
+        dfs = [get_parameter_df(x) for x in parameter_file if x]
 
-    if isinstance(parameter_file, list):
-        parameter_df = pd.concat([pd.read_csv(subset_file, sep='\t',
-                                              float_precision='round_trip')
-                                  for subset_file in parameter_file])
-        # Remove identical parameter definitions
-        parameter_df.drop_duplicates(inplace=True, ignore_index=True)
+        if not dfs:
+            return None
+
+        parameter_df = pd.concat(dfs)
         # Check for contradicting parameter definitions
-        parameter_duplicates = set(parameter_df[PARAMETER_ID].loc[
-            parameter_df[PARAMETER_ID].duplicated()])
-        if parameter_duplicates:
-            raise ValueError(
-                f'The values of {PARAMETER_ID} must be unique or'
-                ' identical between all parameter subset files. The'
-                ' following duplicates were found:\n'
-                f'{parameter_duplicates}'
-            )
+        _check_for_contradicting_parameter_definitions(parameter_df)
+
+        return parameter_df
 
     lint.assert_no_leading_trailing_whitespace(
         parameter_df.columns.values, "parameter")
@@ -75,20 +73,35 @@ def get_parameter_df(
 
     try:
         parameter_df.set_index([PARAMETER_ID], inplace=True)
-    except KeyError:
+    except KeyError as e:
         raise KeyError(
-            f"Parameter table missing mandatory field {PARAMETER_ID}.")
+            f"Parameter table missing mandatory field {PARAMETER_ID}.") from e
+    _check_for_contradicting_parameter_definitions(parameter_df)
 
     return parameter_df
 
 
-def write_parameter_df(df: pd.DataFrame, filename: str) -> None:
+def _check_for_contradicting_parameter_definitions(parameter_df: pd.DataFrame):
+    """
+    Raises a ValueError for non-unique parameter IDs
+    """
+    parameter_duplicates = set(parameter_df.index.values[
+                                    parameter_df.index.duplicated()])
+    if parameter_duplicates:
+        raise ValueError(
+            f'The values of `{PARAMETER_ID}` must be unique. The '
+            f'following duplicates were found:\n{parameter_duplicates}'
+        )
+
+
+def write_parameter_df(df: pd.DataFrame, filename: Union[str, Path]) -> None:
     """Write PEtab parameter table
 
     Arguments:
         df: PEtab parameter table
         filename: Destination file name
     """
+    df = get_parameter_df(df)
     df.to_csv(filename, sep='\t', index=True)
 
 
@@ -122,29 +135,33 @@ def get_optimization_parameter_scaling(
     return dict(zip(estimated_df.index, estimated_df[PARAMETER_SCALE]))
 
 
-def create_parameter_df(sbml_model: libsbml.Model,
-                        condition_df: pd.DataFrame,
-                        observable_df: pd.DataFrame,
-                        measurement_df: pd.DataFrame,
-                        include_optional: bool = False,
-                        parameter_scale: str = LOG10,
-                        lower_bound: Iterable = None,
-                        upper_bound: Iterable = None) -> pd.DataFrame:
+def create_parameter_df(
+        sbml_model: Optional[libsbml.Model] = None,
+        condition_df: Optional[pd.DataFrame] = None,
+        observable_df: Optional[pd.DataFrame] = None,
+        measurement_df: Optional[pd.DataFrame] = None,
+        model: Optional[Model] = None,
+        include_optional: bool = False,
+        parameter_scale: str = LOG10,
+        lower_bound: Iterable = None,
+        upper_bound: Iterable = None
+) -> pd.DataFrame:
     """Create a new PEtab parameter table
 
     All table entries can be provided as string or list-like with length
     matching the number of parameters
 
     Arguments:
-        sbml_model: SBML Model
+        sbml_model: SBML Model (deprecated, mutually exclusive with ``model``)
+        model: PEtab model (mutually exclusive with ``sbml_model``)
         condition_df: PEtab condition DataFrame
         observable_df: PEtab observable DataFrame
         measurement_df: PEtab measurement DataFrame
         include_optional: By default this only returns parameters that are
-            required to be present in the parameter table. If set to True,
+            required to be present in the parameter table. If set to ``True``,
             this returns all parameters that are allowed to be present in the
             parameter table (i.e. also including parameters specified in the
-            SBML model).
+            model).
         parameter_scale: parameter scaling
         lower_bound: lower bound for parameter value
         upper_bound: upper bound for parameter value
@@ -152,14 +169,23 @@ def create_parameter_df(sbml_model: libsbml.Model,
     Returns:
         The created parameter DataFrame
     """
-
+    if sbml_model:
+        warnings.warn("Passing a model via the `sbml_model` argument is "
+                      "deprecated, use `model=petab.models.sbml_model."
+                      "SbmlModel(...)` instead.", DeprecationWarning,
+                      stacklevel=2)
+        from petab.models.sbml_model import SbmlModel
+        if model:
+            raise ValueError("Arguments `model` and `sbml_model` are "
+                             "mutually exclusive.")
+        model = SbmlModel(sbml_model=sbml_model)
     if include_optional:
         parameter_ids = list(get_valid_parameters_for_parameter_table(
-            sbml_model=sbml_model, condition_df=condition_df,
+            model=model, condition_df=condition_df,
             observable_df=observable_df, measurement_df=measurement_df))
     else:
         parameter_ids = list(get_required_parameters_for_parameter_table(
-            sbml_model=sbml_model, condition_df=condition_df,
+            model=model, condition_df=condition_df,
             observable_df=observable_df, measurement_df=measurement_df))
 
     df = pd.DataFrame(
@@ -178,12 +204,11 @@ def create_parameter_df(sbml_model: libsbml.Model,
         })
     df.set_index([PARAMETER_ID], inplace=True)
 
-    # For SBML model parameters, set nominal values as defined in the model
+    # For model parameters, set nominal values as defined in the model
     for parameter_id in df.index:
         try:
-            parameter = sbml_model.getParameter(parameter_id)
-            if parameter:
-                df.loc[parameter_id, NOMINAL_VALUE] = parameter.getValue()
+            df.loc[parameter_id, NOMINAL_VALUE] = \
+                model.get_parameter_value(parameter_id)
         except ValueError:
             # parameter was introduced as condition-specific override and
             # is potentially not present in the model
@@ -192,7 +217,7 @@ def create_parameter_df(sbml_model: libsbml.Model,
 
 
 def get_required_parameters_for_parameter_table(
-        sbml_model: libsbml.Model,
+        model: Model,
         condition_df: pd.DataFrame,
         observable_df: pd.DataFrame,
         measurement_df: pd.DataFrame) -> Set[str]:
@@ -200,7 +225,7 @@ def get_required_parameters_for_parameter_table(
     Get set of parameters which need to go into the parameter table
 
     Arguments:
-        sbml_model: PEtab SBML model
+        model: PEtab model
         condition_df: PEtab condition table
         observable_df: PEtab observable table
         measurement_df: PEtab measurement table
@@ -209,7 +234,7 @@ def get_required_parameters_for_parameter_table(
         Set of parameter IDs which PEtab requires to be present in the
         parameter table. That is all {observable,noise}Parameters from the
         measurement table as well as all parametric condition table overrides
-        that are not defined in the SBML model.
+        that are not defined in the model.
     """
 
     # use ordered dict as proxy for ordered set
@@ -232,32 +257,40 @@ def get_required_parameters_for_parameter_table(
     for kwargs in [dict(observables=True, noise=False),
                    dict(observables=False, noise=True)]:
         output_parameters = observables.get_output_parameters(
-            observable_df, sbml_model, **kwargs)
+            observable_df, model, **kwargs)
         placeholders = observables.get_placeholders(
             observable_df, **kwargs)
         for p in output_parameters:
-            if p not in placeholders and sbml_model.getParameter(p) is None:
+            if p not in placeholders:
                 parameter_ids[p] = None
 
     # Add condition table parametric overrides unless already defined in the
-    # SBML model
+    #  model
     for p in conditions.get_parametric_overrides(condition_df):
-        if sbml_model.getParameter(p) is None:
+        if not model.has_entity_with_id(p):
             parameter_ids[p] = None
 
+    # remove parameters that occur in the condition table and are overridden
+    #  for ALL conditions
+    for p in condition_df.columns[~condition_df.isnull().any()]:
+        try:
+            del parameter_ids[p]
+        except KeyError:
+            pass
     return parameter_ids.keys()
 
 
 def get_valid_parameters_for_parameter_table(
-        sbml_model: libsbml.Model,
+        model: Model,
         condition_df: pd.DataFrame,
         observable_df: pd.DataFrame,
-        measurement_df: pd.DataFrame) -> Set[str]:
+        measurement_df: pd.DataFrame,
+) -> Set[str]:
     """
     Get set of parameters which may be present inside the parameter table
 
     Arguments:
-        sbml_model: PEtab SBML model
+        model: PEtab model
         condition_df: PEtab condition table
         observable_df: PEtab observable table
         measurement_df: PEtab measurement table
@@ -266,38 +299,32 @@ def get_valid_parameters_for_parameter_table(
         Set of parameter IDs which PEtab allows to be present in the
         parameter table.
     """
-
-    # - grab all model parameters
+    # - grab all allowed model parameters
     # - grab all output parameters defined in {observable,noise}Formula
     # - grab all parameters from measurement table
     # - grab all parametric overrides from condition table
     # - remove parameters for which condition table columns exist
-    # - remove observables assigment targets
-    # - remove sigma assignment targets
     # - remove placeholder parameters
     #   (only partial overrides are not supported)
 
     placeholders = set(observables.get_placeholders(observable_df))
 
-    # exclude rule targets
-    assignment_targets = {ar.getVariable()
-                          for ar in sbml_model.getListOfRules()}
-
     # must not go into parameter table
     blackset = set()
     # collect assignment targets
     blackset |= placeholders
-    blackset |= assignment_targets
     blackset |= set(condition_df.columns.values) - {CONDITION_NAME}
 
-    # use ordered dict as proxy for ordered set
+    # don't use sets here, to have deterministic ordering,
+    #  e.g. for creating parameter tables
     parameter_ids = OrderedDict.fromkeys(
-        p.getId() for p in sbml_model.getListOfParameters()
-        if p.getId() not in blackset)
+        p for p in model.get_valid_parameters_for_parameter_table()
+        if p not in blackset
+    )
 
     # add output parameters from observables table
     output_parameters = observables.get_output_parameters(
-        observable_df, sbml_model)
+        observable_df=observable_df, model=model)
     for p in output_parameters:
         if p not in blackset:
             parameter_ids[p] = None
@@ -323,13 +350,15 @@ def get_valid_parameters_for_parameter_table(
     return parameter_ids.keys()
 
 
-def get_priors_from_df(parameter_df: pd.DataFrame,
-                       mode: str) -> List[Tuple]:
+def get_priors_from_df(
+        parameter_df: pd.DataFrame,
+        mode: Literal['initialization', 'objective'],
+) -> List[Tuple]:
     """Create list with information about the parameter priors
 
     Arguments:
         parameter_df: PEtab parameter table
-        mode: 'initialization' or 'objective'
+        mode: ``'initialization'`` or ``'objective'``
 
     Returns:
         List with prior information.
@@ -350,8 +379,10 @@ def get_priors_from_df(parameter_df: pd.DataFrame,
         if core.is_empty(pars_str):
             lb, ub = map_scale([row[LOWER_BOUND], row[UPPER_BOUND]],
                                [row[PARAMETER_SCALE]] * 2)
-            pars_str = f'{lb};{ub}'
-        prior_pars = tuple([float(entry) for entry in pars_str.split(';')])
+            pars_str = f'{lb}{PARAMETER_SEPARATOR}{ub}'
+        prior_pars = tuple(
+            float(entry) for entry in pars_str.split(PARAMETER_SEPARATOR)
+        )
 
         # add parameter scale and bounds, as this may be needed
         par_scale = row[PARAMETER_SCALE]
@@ -368,55 +399,59 @@ def get_priors_from_df(parameter_df: pd.DataFrame,
     return prior_list
 
 
-def scale(parameter: numbers.Number, scale_str: 'str') -> numbers.Number:
-    """Scale parameter according to `scale_str`.
+def scale(
+        parameter: numbers.Number,
+        scale_str: PARAMETER_SCALE_ARGS,
+) -> numbers.Number:
+    """Scale parameter according to ``scale_str``.
 
     Arguments:
         parameter:
             Parameter to be scaled.
         scale_str:
-            One of 'lin' (synonymous with ''), 'log', 'log10'.
+            One of ``'lin'`` (synonymous with ``''``), ``'log'``, ``'log10'``.
 
     Returns:
         The scaled parameter.
     """
-
     if scale_str == LIN or not scale_str:
         return parameter
     if scale_str == LOG:
         return np.log(parameter)
     if scale_str == LOG10:
         return np.log10(parameter)
-    raise ValueError("Invalid parameter scaling: " + scale_str)
+    raise ValueError(f"Invalid parameter scaling: {scale_str}")
 
 
-def unscale(parameter: numbers.Number, scale_str: 'str') -> numbers.Number:
-    """Unscale parameter according to `scale_str`.
+def unscale(
+        parameter: numbers.Number,
+        scale_str: PARAMETER_SCALE_ARGS,
+) -> numbers.Number:
+    """Unscale parameter according to ``scale_str``.
 
     Arguments:
         parameter:
             Parameter to be unscaled.
         scale_str:
-            One of 'lin' (synonymous with ''), 'log', 'log10'.
+            One of ``'lin'`` (synonymous with ``''``), ``'log'``, ``'log10'``.
 
     Returns:
         The unscaled parameter.
     """
-
     if scale_str == LIN or not scale_str:
         return parameter
     if scale_str == LOG:
         return np.exp(parameter)
     if scale_str == LOG10:
         return 10**parameter
-    raise ValueError("Invalid parameter scaling: " + scale_str)
+    raise ValueError(f"Invalid parameter scaling: {scale_str}")
 
 
 def map_scale(
-    parameters: Iterable[numbers.Number],
-    scale_strs: Union[Iterable[str], str]
+    parameters: Sequence[numbers.Number],
+    scale_strs: Union[Iterable[PARAMETER_SCALE_ARGS], PARAMETER_SCALE_ARGS],
 ) -> Iterable[numbers.Number]:
-    """Scale the parameters, i.e. as `scale()`, but for Iterables.
+    """Scale the parameters, i.e. as :func:`scale`, but for Sequences.
 
     Arguments:
         parameters:
@@ -433,10 +468,10 @@ def map_scale(
 
 
 def map_unscale(
-    parameters: Iterable[numbers.Number],
-    scale_strs: Union[Iterable[str], str]
+    parameters: Sequence[numbers.Number],
+    scale_strs: Union[Iterable[PARAMETER_SCALE_ARGS], PARAMETER_SCALE_ARGS],
 ) -> Iterable[numbers.Number]:
-    """Unscale the parameters, i.e. as `unscale()`, but for Iterables.
+    """Unscale the parameters, i.e. as :func:`unscale`, but for Sequences.
 
     Arguments:
         parameters:
@@ -480,6 +515,6 @@ def normalize_parameter_df(parameter_df: pd.DataFrame) -> pd.DataFrame:
                     and row[prior_type_col] == PARAMETER_SCALE_UNIFORM:
                 lb, ub = map_scale([row[LOWER_BOUND], row[UPPER_BOUND]],
                                    [row[PARAMETER_SCALE]] * 2)
-                df.loc[irow, prior_par_col] = f'{lb};{ub}'
+                df.loc[irow, prior_par_col] = f'{lb}{PARAMETER_SEPARATOR}{ub}'
 
     return df
