@@ -1,29 +1,34 @@
 """PEtab Problem class"""
+from __future__ import annotations
 
 import os
-# Renamed to `Path_` to avoid unknown error during Sphinx doc build
-from pathlib import Path as Path_
 import tempfile
-from typing import Dict, Iterable, List, Optional, Union
+from pathlib import Path, PurePosixPath
+from typing import Dict, Iterable, List, Optional, Union, TYPE_CHECKING
+from urllib.parse import unquote, urlparse, urlunparse
 from warnings import warn
 
 import pandas as pd
-import libsbml
-from . import (parameter_mapping, measurements, conditions, parameters,
-               sampling, sbml, yaml, core, observables, format_version)
-from .C import *  # noqa: F403
 
-__all__ = ['Problem', 'get_default_condition_file_name',
-           'get_default_measurement_file_name',
-           'get_default_parameter_file_name',
-           'get_default_sbml_file_name']
+from . import (conditions, core, format_version, measurements, observables,
+               parameter_mapping, parameters, sampling, sbml, yaml)
+from .C import *  # noqa: F403
+from .models import MODEL_TYPE_SBML
+from .models.model import Model, model_factory
+from .models.sbml_model import SbmlModel
+
+if TYPE_CHECKING:
+    import libsbml
+
+
+__all__ = ['Problem']
 
 
 class Problem:
     """
     PEtab parameter estimation problem as defined by
 
-    - SBML model
+    - model
     - condition table
     - measurement table
     - parameter table
@@ -37,65 +42,108 @@ class Problem:
         parameter_df: PEtab parameter table
         observable_df: PEtab observable table
         visualization_df: PEtab visualization table
-        sbml_reader: Stored to keep object alive.
-        sbml_document: Stored to keep object alive.
-        sbml_model: PEtab SBML model
+        model: The underlying model
+        sbml_reader: Stored to keep object alive (deprecated).
+        sbml_document: Stored to keep object alive (deprecated).
+        sbml_model: PEtab SBML model (deprecated)
+        extensions_config: Information on the extensions used
     """
 
-    def __init__(self,
-                 sbml_model: libsbml.Model = None,
-                 sbml_reader: libsbml.SBMLReader = None,
-                 sbml_document: libsbml.SBMLDocument = None,
-                 condition_df: pd.DataFrame = None,
-                 measurement_df: pd.DataFrame = None,
-                 parameter_df: pd.DataFrame = None,
-                 visualization_df: pd.DataFrame = None,
-                 observable_df: pd.DataFrame = None):
-
+    def __init__(
+            self,
+            sbml_model: libsbml.Model = None,
+            sbml_reader: libsbml.SBMLReader = None,
+            sbml_document: libsbml.SBMLDocument = None,
+            model: Model = None,
+            condition_df: pd.DataFrame = None,
+            measurement_df: pd.DataFrame = None,
+            parameter_df: pd.DataFrame = None,
+            visualization_df: pd.DataFrame = None,
+            observable_df: pd.DataFrame = None,
+            extensions_config: Dict = None,
+    ):
         self.condition_df: Optional[pd.DataFrame] = condition_df
         self.measurement_df: Optional[pd.DataFrame] = measurement_df
         self.parameter_df: Optional[pd.DataFrame] = parameter_df
         self.visualization_df: Optional[pd.DataFrame] = visualization_df
         self.observable_df: Optional[pd.DataFrame] = observable_df
 
-        self.sbml_reader: Optional[libsbml.SBMLReader] = sbml_reader
-        self.sbml_document: Optional[libsbml.SBMLDocument] = sbml_document
-        self.sbml_model: Optional[libsbml.Model] = sbml_model
+        if any((sbml_model, sbml_document, sbml_reader),):
+            warn("Passing `sbml_model`, `sbml_document`, or `sbml_reader` "
+                 "to petab.Problem is deprecated and will be removed in a "
+                 "future version. Use `model=petab.models.sbml_model."
+                 "SbmlModel(...)` instead.", DeprecationWarning, stacklevel=2)
+            if model:
+                raise ValueError("Must only provide one of (`sbml_model`, "
+                                 "`sbml_document`, `sbml_reader`) or `model`.")
 
-    def __getstate__(self):
-        """Return state for pickling"""
-        state = self.__dict__.copy()
+            model = SbmlModel(
+                sbml_model=sbml_model,
+                sbml_reader=sbml_reader,
+                sbml_document=sbml_document)
 
-        # libsbml stuff cannot be serialized directly
-        if self.sbml_model:
-            sbml_document = self.sbml_model.getSBMLDocument()
-            sbml_writer = libsbml.SBMLWriter()
-            state['sbml_string'] = sbml_writer.writeSBMLToString(sbml_document)
+        self.model: Optional[Model] = model
+        self.extensions_config = extensions_config or {}
 
-        exclude = ['sbml_reader', 'sbml_document', 'sbml_model']
-        for key in exclude:
-            state.pop(key)
+    def __getattr__(self, name):
+        # For backward-compatibility, allow access to SBML model related
+        #  attributes now stored in self.model
+        if name in {'sbml_model', 'sbml_reader', 'sbml_document'}:
+            return getattr(self.model, name) if self.model else None
+        raise AttributeError(f"'{self.__class__.__name__}' object has no "
+                             f"attribute '{name}'")
 
-        return state
+    def __setattr__(self, name, value):
+        # For backward-compatibility, allow access to SBML model related
+        #  attributes now stored in self.model
+        if name in {'sbml_model', 'sbml_reader', 'sbml_document'}:
+            if self.model:
+                setattr(self.model, name, value)
+            else:
+                self.model = SbmlModel(**{name: value})
+        else:
+            super().__setattr__(name, value)
 
-    def __setstate__(self, state):
-        """Set state after unpickling"""
-        # load SBML model from pickled string
-        sbml_string = state.pop('sbml_string', None)
-        if sbml_string:
-            self.sbml_reader, self.sbml_document, self.sbml_model = \
-                sbml.load_sbml_from_string(sbml_string)
+    def __str__(self):
+        model = f"with model ({self.model})" if self.model else "without model"
+        conditions = f"{self.condition_df.shape[0]} conditions" \
+            if self.condition_df is not None else "without conditions table"
 
-        self.__dict__.update(state)
+        observables = f"{self.observable_df.shape[0]} observables" \
+            if self.observable_df is not None else "without observables table"
+
+        measurements = f"{self.measurement_df.shape[0]} measurements" \
+            if self.measurement_df is not None \
+            else "without measurements table"
+
+        if self.parameter_df is not None:
+            num_estimated_parameters = sum(self.parameter_df[ESTIMATE] == 1) \
+                if ESTIMATE in self.parameter_df \
+                else self.parameter_df.shape[0]
+            parameters = f"{num_estimated_parameters} estimated parameters"
+        else:
+            parameters = "without parameter_df table"
+
+        return (
+            f"PEtab Problem {model}, {conditions}, {observables}, "
+            f"{measurements}, {parameters}"
+        )
 
     @staticmethod
-    def from_files(sbml_file: str = None,
-                   condition_file: str = None,
-                   measurement_file: Union[str, Iterable[str]] = None,
-                   parameter_file: Union[str, List[str]] = None,
-                   visualization_files: Union[str, Iterable[str]] = None,
-                   observable_files: Union[str, Iterable[str]] = None
-                   ) -> 'Problem':
+    def from_files(
+            sbml_file: Union[str, Path] = None,
+            condition_file:
+            Union[str, Path, Iterable[Union[str, Path]]] = None,
+            measurement_file: Union[str, Path,
+                                    Iterable[Union[str, Path]]] = None,
+            parameter_file: Union[str, Path,
+                                  Iterable[Union[str, Path]]] = None,
+            visualization_files: Union[str, Path,
+                                       Iterable[Union[str, Path]]] = None,
+            observable_files: Union[str, Path,
+                                    Iterable[Union[str, Path]]] = None,
+            extensions_config: Dict = None,
+    ) -> 'Problem':
         """
         Factory method to load model and tables from files.
 
@@ -106,59 +154,84 @@ class Problem:
             parameter_file: PEtab parameter table
             visualization_files: PEtab visualization tables
             observable_files: PEtab observables tables
+            model_id: PEtab ID of the model
+            extensions_config: Information on the extensions used
         """
+        warn("petab.Problem.from_files is deprecated and will be removed in a "
+             "future version. Use `petab.Problem.from_yaml instead.",
+             DeprecationWarning, stacklevel=2)
 
-        sbml_model = sbml_document = sbml_reader = None
-        condition_df = measurement_df = parameter_df = visualization_df = None
-        observable_df = None
+        model = model_factory(sbml_file, MODEL_TYPE_SBML) \
+            if sbml_file else None
 
-        if condition_file:
-            condition_df = conditions.get_condition_df(condition_file)
+        condition_df = core.concat_tables(
+            condition_file, conditions.get_condition_df) \
+            if condition_file else None
 
-        if measurement_file:
-            # If there are multiple tables, we will merge them
-            measurement_df = core.concat_tables(
-                measurement_file, measurements.get_measurement_df)
+        # If there are multiple tables, we will merge them
+        measurement_df = core.concat_tables(
+            measurement_file, measurements.get_measurement_df) \
+            if measurement_file else None
 
-        if parameter_file:
-            parameter_df = parameters.get_parameter_df(parameter_file)
+        parameter_df = parameters.get_parameter_df(parameter_file) \
+            if parameter_file else None
 
-        if sbml_file:
-            sbml_reader, sbml_document, sbml_model = \
-                sbml.get_sbml_model(sbml_file)
+        # If there are multiple tables, we will merge them
+        visualization_df = core.concat_tables(
+            visualization_files, core.get_visualization_df) \
+            if visualization_files else None
 
-        if visualization_files:
-            # If there are multiple tables, we will merge them
-            visualization_df = core.concat_tables(
-                visualization_files, core.get_visualization_df)
+        # If there are multiple tables, we will merge them
+        observable_df = core.concat_tables(
+            observable_files, observables.get_observable_df) \
+            if observable_files else None
 
-        if observable_files:
-            # If there are multiple tables, we will merge them
-            observable_df = core.concat_tables(
-                observable_files, observables.get_observable_df)
-
-        return Problem(condition_df=condition_df,
-                       measurement_df=measurement_df,
-                       parameter_df=parameter_df,
-                       observable_df=observable_df,
-                       sbml_model=sbml_model,
-                       sbml_document=sbml_document,
-                       sbml_reader=sbml_reader,
-                       visualization_df=visualization_df)
+        return Problem(
+            model=model,
+            condition_df=condition_df,
+            measurement_df=measurement_df,
+            parameter_df=parameter_df,
+            observable_df=observable_df,
+            visualization_df=visualization_df,
+            extensions_config=extensions_config,
+        )
 
     @staticmethod
-    def from_yaml(yaml_config: Union[Dict, str]) -> 'Problem':
+    def from_yaml(yaml_config: Union[Dict, Path, str]) -> 'Problem':
         """
         Factory method to load model and tables as specified by YAML file.
 
         Arguments:
             yaml_config: PEtab configuration as dictionary or YAML file name
         """
+        if isinstance(yaml_config, Path):
+            yaml_config = str(yaml_config)
+
+        get_path = lambda filename: filename  # noqa: E731
         if isinstance(yaml_config, str):
-            path_prefix = os.path.dirname(yaml_config)
+            yaml_path = yaml_config
             yaml_config = yaml.load_yaml(yaml_config)
-        else:
-            path_prefix = ""
+
+            # yaml_config may be path or URL
+            path_url = urlparse(yaml_path)
+            if not path_url.scheme or \
+                    (path_url.scheme != 'file' and not path_url.netloc):
+                # a regular file path string
+                path_prefix = Path(yaml_path).parent
+                get_path = lambda filename: \
+                    path_prefix / filename  # noqa: E731
+            else:
+                # a URL
+                # extract parent path from
+                url_path = unquote(urlparse(yaml_path).path)
+                parent_path = str(PurePosixPath(url_path).parent)
+                path_prefix = urlunparse(
+                    (path_url.scheme, path_url.netloc, parent_path,
+                     path_url.params, path_url.query, path_url.fragment)
+                )
+                # need "/" on windows, not "\"
+                get_path = lambda filename: \
+                    f"{path_prefix}/{filename}"  # noqa: E731
 
         if yaml.is_composite_problem(yaml_config):
             raise ValueError('petab.Problem.from_yaml() can only be used for '
@@ -173,71 +246,65 @@ class Problem:
 
         problem0 = yaml_config['problems'][0]
 
-        yaml.assert_single_condition_and_sbml_file(problem0)
+        if len(problem0[SBML_FILES]) > 1:
+            # TODO https://github.com/PEtab-dev/libpetab-python/issues/6
+            raise NotImplementedError(
+                'Support for multiple models is not yet implemented.')
 
         if isinstance(yaml_config[PARAMETER_FILE], list):
-            parameter_file = [
-                os.path.join(path_prefix, f)
+            parameter_df = parameters.get_parameter_df([
+                get_path(f)
                 for f in yaml_config[PARAMETER_FILE]
-            ]
+            ])
         else:
-            parameter_file = os.path.join(
-                path_prefix, yaml_config[PARAMETER_FILE])
+            parameter_df = parameters.get_parameter_df(
+                get_path(yaml_config[PARAMETER_FILE])) \
+                if yaml_config[PARAMETER_FILE] else None
 
-        return Problem.from_files(
-            sbml_file=os.path.join(path_prefix, problem0[SBML_FILES][0]),
-            measurement_file=[os.path.join(path_prefix, f)
-                              for f in problem0[MEASUREMENT_FILES]],
-            condition_file=os.path.join(
-                path_prefix, problem0[CONDITION_FILES][0]),
-            parameter_file=parameter_file,
-            visualization_files=[
-                os.path.join(path_prefix, f)
-                for f in problem0.get(VISUALIZATION_FILES, [])],
-            observable_files=[
-                os.path.join(path_prefix, f)
-                for f in problem0.get(OBSERVABLE_FILES, [])]
+        model = model_factory(get_path(problem0[SBML_FILES][0]),
+                              MODEL_TYPE_SBML) \
+            if problem0[SBML_FILES] else None
+
+        measurement_files = [
+            get_path(f) for f in problem0.get(MEASUREMENT_FILES, [])]
+        # If there are multiple tables, we will merge them
+        measurement_df = core.concat_tables(
+            measurement_files, measurements.get_measurement_df) \
+            if measurement_files else None
+
+        condition_files = [
+            get_path(f) for f in problem0.get(CONDITION_FILES, [])]
+        # If there are multiple tables, we will merge them
+        condition_df = core.concat_tables(
+            condition_files, conditions.get_condition_df) \
+            if condition_files else None
+
+        visualization_files = [
+            get_path(f) for f in problem0.get(VISUALIZATION_FILES, [])]
+        # If there are multiple tables, we will merge them
+        visualization_df = core.concat_tables(
+            visualization_files, core.get_visualization_df) \
+            if visualization_files else None
+
+        observable_files = [
+            get_path(f) for f in problem0.get(OBSERVABLE_FILES, [])]
+        # If there are multiple tables, we will merge them
+        observable_df = core.concat_tables(
+            observable_files, observables.get_observable_df) \
+            if observable_files else None
+
+        return Problem(
+            condition_df=condition_df,
+            measurement_df=measurement_df,
+            parameter_df=parameter_df,
+            observable_df=observable_df,
+            model=model,
+            visualization_df=visualization_df,
+            extensions_config=yaml_config.get(EXTENSIONS, {})
         )
 
     @staticmethod
-    def from_folder(folder: str, model_name: str = None) -> 'Problem':
-        """
-        Factory method to use the standard folder structure
-        and file names, i.e.
-
-        ::
-
-            ${model_name}/
-              +-- experimentalCondition_${model_name}.tsv
-              +-- measurementData_${model_name}.tsv
-              +-- model_${model_name}.xml
-              +-- parameters_${model_name}.tsv
-
-        Arguments:
-            folder:
-                Path to the directory in which the files are located.
-            model_name:
-                If specified, overrides the model component in the file names.
-                Defaults to the last component of ``folder``.
-        """
-        warn("This function will be removed in future releases. "
-             "Consider using a PEtab YAML file for grouping files",
-             DeprecationWarning)
-
-        folder = os.path.abspath(folder)
-        if model_name is None:
-            model_name = os.path.split(folder)[-1]
-
-        return Problem.from_files(
-            condition_file=get_default_condition_file_name(model_name, folder),
-            measurement_file=get_default_measurement_file_name(model_name,
-                                                               folder),
-            parameter_file=get_default_parameter_file_name(model_name, folder),
-            sbml_file=get_default_sbml_file_name(model_name, folder),
-        )
-
-    @staticmethod
-    def from_combine(filename: str) -> 'Problem':
+    def from_combine(filename: Union[Path, str]) -> 'Problem':
         """Read PEtab COMBINE archive (http://co.mbine.org/documents/archive).
 
         See also :py:func:`petab.create_combine_archive`.
@@ -252,13 +319,13 @@ class Problem:
         # other SWIG interfaces
         try:
             import libcombine
-        except ImportError:
+        except ImportError as e:
             raise ImportError(
                 "To use PEtab's COMBINE functionality, libcombine "
-                "(python-libcombine) must be installed.")
+                "(python-libcombine) must be installed.") from e
 
         archive = libcombine.CombineArchive()
-        if archive.initializeFromArchive(filename) is None:
+        if archive.initializeFromArchive(str(filename)) is None:
             print(f"Invalid Combine Archive: {filename}")
             return None
 
@@ -273,8 +340,8 @@ class Problem:
 
     def to_files_generic(
         self,
-        prefix_path: Union[str, Path_],
-    ) -> None:
+        prefix_path: Union[str, Path],
+    ) -> str:
         """Save a PEtab problem to generic file names.
 
         The PEtab problem YAML file is always created. PEtab data files are
@@ -286,13 +353,13 @@ class Problem:
             prefix_path:
                 Specify a prefix to all paths, to avoid specifying the
                 prefix for all paths individually. NB: the prefix is added to
-                paths before `relative_paths` is handled downstream in
-                `petab.yaml.create_problem_yaml`.
+                paths before ``relative_paths`` is handled downstream in
+                :func:`petab.yaml.create_problem_yaml`.
 
         Returns:
             The path to the PEtab problem YAML file.
         """
-        prefix_path = Path_(prefix_path)
+        prefix_path = Path(prefix_path)
 
         # Generate generic filenames for data tables in the PEtab problem that
         # contain data.
@@ -307,7 +374,10 @@ class Problem:
             if getattr(self, f'{table_name}_df') is not None:
                 filenames[f'{table_name}_file'] = f'{table_name}s.tsv'
 
-        if self.sbml_document is not None:
+        if self.model:
+            if not isinstance(self.model, SbmlModel):
+                raise NotImplementedError("Saving non-SBML models is "
+                                          "currently not supported.")
             filenames['sbml_file'] = 'model.xml'
 
         filenames['yaml_file'] = 'problem.yaml'
@@ -318,16 +388,19 @@ class Problem:
             return filenames['yaml_file']
         return str(prefix_path / filenames['yaml_file'])
 
-    def to_files(self,
-                 sbml_file: Optional[str] = None,
-                 condition_file: Optional[str] = None,
-                 measurement_file: Optional[str] = None,
-                 parameter_file: Optional[str] = None,
-                 visualization_file: Optional[str] = None,
-                 observable_file: Optional[str] = None,
-                 yaml_file: Optional[str] = None,
-                 prefix_path: Optional[Union[str, Path_]] = None,
-                 relative_paths: bool = True,) -> None:
+    def to_files(
+            self,
+            sbml_file: Union[None, str, Path] = None,
+            condition_file: Union[None, str, Path] = None,
+            measurement_file: Union[None, str, Path] = None,
+            parameter_file: Union[None, str, Path] = None,
+            visualization_file: Union[None, str, Path] = None,
+            observable_file: Union[None, str, Path] = None,
+            yaml_file: Union[None, str, Path] = None,
+            prefix_path: Union[None, str, Path] = None,
+            relative_paths: bool = True,
+            model_file: Union[None, str, Path] = None,
+    ) -> None:
         """
         Write PEtab tables to files for this problem
 
@@ -338,7 +411,8 @@ class Problem:
         visualization tables, they will be merged and written to a single file.
 
         Arguments:
-            sbml_file: SBML model destination
+            sbml_file: SBML model destination (deprecated)
+            model_file: Model destination
             condition_file: Condition table destination
             measurement_file: Measurement table destination
             parameter_file: Parameter table destination
@@ -348,25 +422,34 @@ class Problem:
             prefix_path:
                 Specify a prefix to all paths, to avoid specifying the
                 prefix for all paths individually. NB: the prefix is added to
-                paths before `relative_paths` is handled.
+                paths before ``relative_paths`` is handled.
             relative_paths:
                 whether all paths in the YAML file should be
-                relative to the location of the YAML file. If `False`, then
+                relative to the location of the YAML file. If ``False``, then
                 paths are left unchanged.
 
         Raises:
             ValueError:
                 If a destination was provided for a non-existing entity.
         """
+        if sbml_file:
+            warn("The `sbml_file` argument is deprecated and will be "
+                 "removed in a future version. Use `model_file` instead.",
+                 DeprecationWarning, stacklevel=2)
+
+            if model_file:
+                raise ValueError("Must provide either `sbml_file` or "
+                                 "`model_file` argument, but not both.")
+
+            model_file = sbml_file
+
         if prefix_path is not None:
-            prefix_path = Path_(prefix_path)
+            prefix_path = Path(prefix_path)
 
-            def add_prefix(path0: Union[None, str, Path_]) -> str:
-                if path0 is None:
-                    return path0
-                return str(prefix_path / path0)
+            def add_prefix(path0: Union[None, str, Path]) -> str:
+                return path0 if path0 is None else str(prefix_path / path0)
 
-            sbml_file = add_prefix(sbml_file)
+            model_file = add_prefix(model_file)
             condition_file = add_prefix(condition_file)
             measurement_file = add_prefix(measurement_file)
             parameter_file = add_prefix(parameter_file)
@@ -374,12 +457,8 @@ class Problem:
             visualization_file = add_prefix(visualization_file)
             yaml_file = add_prefix(yaml_file)
 
-        if sbml_file:
-            if self.sbml_document is not None:
-                sbml.write_sbml(self.sbml_document, sbml_file)
-            else:
-                raise ValueError("Unable to save SBML model with no "
-                                 "sbml_doc set.")
+        if model_file:
+            self.model.to_file(model_file)
 
         def error(name: str) -> ValueError:
             return ValueError(f"Unable to save non-existent {name} table")
@@ -420,7 +499,7 @@ class Problem:
                 raise error("visualization")
 
         if yaml_file:
-            yaml.create_problem_yaml(sbml_file, condition_file,
+            yaml.create_problem_yaml(model_file, condition_file,
                                      measurement_file, parameter_file,
                                      observable_file, yaml_file,
                                      visualization_file,
@@ -444,45 +523,18 @@ class Problem:
 
     def get_model_parameters(self):
         """See :py:func:`petab.sbml.get_model_parameters`"""
+        warn("petab.Problem.get_model_parameters is deprecated and will be "
+             "removed in a future version.",
+             DeprecationWarning, stacklevel=2)
+
         return sbml.get_model_parameters(self.sbml_model)
 
-    def get_observables(self, remove: bool = False):
-        """
-        Returns dictionary of observables definitions.
-
-        See :py:func:`petab.assignment_rules_to_dict` for details.
-        """
-        warn("This function will be removed in future releases.",
-             DeprecationWarning)
-
-        return sbml.get_observables(sbml_model=self.sbml_model, remove=remove)
-
     def get_observable_ids(self):
         """
         Returns dictionary of observable ids.
         """
         return list(self.observable_df.index)
 
-    def get_sigmas(self, remove: bool = False):
-        """
-        Return dictionary of observableId => sigma as defined in the SBML
-        model.
-
-        This does not include parameter mappings defined in the measurement
-        table.
-        """
-        warn("This function will be removed in future releases.",
-             DeprecationWarning)
-
-        return sbml.get_sigmas(sbml_model=self.sbml_model, remove=remove)
-
-    def get_noise_distributions(self):
-        """
-        See :py:func:`petab.get_noise_distributions`.
-        """
-        return measurements.get_noise_distributions(
-            measurement_df=self.measurement_df)
-
     def _apply_mask(self, v: List, free: bool = True, fixed: bool = True):
         """Apply mask of only free or only fixed values.
 
@@ -685,26 +737,21 @@ class Problem:
         return measurements.get_simulation_conditions(self.measurement_df)
 
     def get_optimization_to_simulation_parameter_mapping(
-            self,
-            warn_unmapped: bool = True,
-            scaled_parameters: bool = False,
-            allow_timepoint_specific_numeric_noise_parameters:
-            bool = False,
+            self, **kwargs
     ):
         """
-        See get_simulation_to_optimization_parameter_mapping.
+        See
+        :py:func:`petab.parameter_mapping.get_optimization_to_simulation_parameter_mapping`,
+        to which all keyword arguments are forwarded.
         """
-        return parameter_mapping\
+        return parameter_mapping \
             .get_optimization_to_simulation_parameter_mapping(
-                self.condition_df,
-                self.measurement_df,
-                self.parameter_df,
-                self.observable_df,
-                self.sbml_model,
-                warn_unmapped=warn_unmapped,
-                scaled_parameters=scaled_parameters,
-                allow_timepoint_specific_numeric_noise_parameters=  # noqa: E251,E501
-                allow_timepoint_specific_numeric_noise_parameters
+                condition_df=self.condition_df,
+                measurement_df=self.measurement_df,
+                parameter_df=self.parameter_df,
+                observable_df=self.observable_df,
+                model=self.model,
+                **kwargs
             )
 
     def create_parameter_df(self, *args, **kwargs):
@@ -713,10 +760,10 @@ class Problem:
         See :py:func:`create_parameter_df`.
         """
         return parameters.create_parameter_df(
-            self.sbml_model,
-            self.condition_df,
-            self.observable_df,
-            self.measurement_df,
+            model=self.model,
+            condition_df=self.condition_df,
+            observable_df=self.observable_df,
+            measurement_df=self.measurement_df,
             *args, **kwargs)
 
     def sample_parameter_startpoints(self, n_starts: int = 100):
@@ -774,35 +821,3 @@ class Problem:
             )
             for parameter_id, parameter_value in x_dict.items()
         }
-
-
-def get_default_condition_file_name(model_name: str, folder: str = ''):
-    """Get file name according to proposed convention"""
-    warn("This function will be removed in future releases. ",
-         DeprecationWarning)
-
-    return os.path.join(folder, f"experimentalCondition_{model_name}.tsv")
-
-
-def get_default_measurement_file_name(model_name: str, folder: str = ''):
-    """Get file name according to proposed convention"""
-    warn("This function will be removed in future releases. ",
-         DeprecationWarning)
-
-    return os.path.join(folder, f"measurementData_{model_name}.tsv")
-
-
-def get_default_parameter_file_name(model_name: str, folder: str = ''):
-    """Get file name according to proposed convention"""
-    warn("This function will be removed in future releases. ",
-         DeprecationWarning)
-
-    return os.path.join(folder, f"parameters_{model_name}.tsv")
-
-
-def get_default_sbml_file_name(model_name: str, folder: str = ''):
-    """Get file name according to proposed convention"""
-    warn("This function will be removed in future releases. ",
-         DeprecationWarning)
-
-    return os.path.join(folder, f"model_{model_name}.xml")
