@@ -33,8 +33,10 @@ class Plotter(ABC):
         self.data_provider = data_provider
 
     @abstractmethod
-    def generate_figure(self, subplot_dir: Optional[str] = None
-                        ) -> Optional[Dict[str, plt.Subplot]]:
+    def generate_figure(
+            self,
+            subplot_dir: Optional[str] = None
+    ) -> Optional[Dict[str, plt.Subplot]]:
         pass
 
 
@@ -45,9 +47,32 @@ class MPLPlotter(Plotter):
     def __init__(self, figure: Figure, data_provider: DataProvider):
         super().__init__(figure, data_provider)
 
-    def generate_lineplot(self, ax: 'matplotlib.pyplot.Axes',
-                          dataplot: DataPlot,
-                          plotTypeData: str) -> None:
+    @staticmethod
+    def _error_column_for_plot_type_data(plot_type_data: str) -> Optional[str]:
+        """Translate PEtab plotTypeData value to column name of internal
+        data representation
+
+        Parameters
+        ----------
+            plot_type_data: PEtab plotTypeData value
+        Returns
+        -------
+            Name of corresponding column
+        """
+        if plot_type_data == MEAN_AND_SD:
+            return 'sd'
+        if plot_type_data == MEAN_AND_SEM:
+            return 'sem'
+        if plot_type_data == PROVIDED:
+            return 'noise_model'
+        return None
+
+    def generate_lineplot(
+            self,
+            ax: 'matplotlib.pyplot.Axes',
+            dataplot: DataPlot,
+            plotTypeData: str
+    ) -> None:
         """
         Generate lineplot.
 
@@ -62,23 +87,16 @@ class MPLPlotter(Plotter):
         plotTypeData:
             Specifies how replicates should be handled.
         """
-
         simu_color = None
         measurements_to_plot, simulations_to_plot = \
             self.data_provider.get_data_to_plot(dataplot,
                                                 plotTypeData == PROVIDED)
-        noise_col = None
-        # set type of noise
-        if plotTypeData == MEAN_AND_SD:
-            noise_col = 'sd'
-        elif plotTypeData == MEAN_AND_SEM:
-            noise_col = 'sem'
-        elif plotTypeData == PROVIDED:
-            noise_col = 'noise_model'
+        noise_col = self._error_column_for_plot_type_data(plotTypeData)
 
         label_base = dataplot.legendEntry
 
-        if measurements_to_plot is not None:
+        if measurements_to_plot is not None \
+                and not measurements_to_plot.data_to_plot.empty:
             # plotting all measurement data
 
             if plotTypeData == REPLICATE:
@@ -139,9 +157,12 @@ class MPLPlotter(Plotter):
                 label=label_base + " simulation", color=simu_color
             )
 
-    def generate_barplot(self, ax: 'matplotlib.pyplot.Axes',
-                         dataplot: DataPlot,
-                         plotTypeData: str) -> None:
+    def generate_barplot(
+            self,
+            ax: 'matplotlib.pyplot.Axes',
+            dataplot: DataPlot,
+            plotTypeData: str
+    ) -> None:
         """
         Generate barplot.
 
@@ -155,16 +176,8 @@ class MPLPlotter(Plotter):
             Specifies how replicates should be handled.
         """
         # TODO: plotTypeData == REPLICATE?
-        # set type of noise
-        noise_col = None
-        if plotTypeData == MEAN_AND_SD:
-            noise_col = 'sd'
-        elif plotTypeData == MEAN_AND_SEM:
-            noise_col = 'sem'
-        elif plotTypeData == PROVIDED:
-            noise_col = 'noise_model'
+        noise_col = self._error_column_for_plot_type_data(plotTypeData)
 
-        simu_colors = None
         measurements_to_plot, simulations_to_plot = \
             self.data_provider.get_data_to_plot(dataplot,
                                                 plotTypeData == PROVIDED)
@@ -185,20 +198,22 @@ class MPLPlotter(Plotter):
         color = plt.rcParams["axes.prop_cycle"].by_key()["color"][0]
 
         if measurements_to_plot is not None:
-            p = ax.bar(x_name, measurements_to_plot.data_to_plot['mean'],
-                       yerr=measurements_to_plot.data_to_plot[noise_col],
-                       color=color, **bar_kwargs, label='measurement')
-            simu_colors = p[0].get_facecolor()
+            ax.bar(x_name, measurements_to_plot.data_to_plot['mean'],
+                   yerr=measurements_to_plot.data_to_plot[noise_col],
+                   color=color, **bar_kwargs, label='measurement')
 
         if simulations_to_plot is not None:
             bar_kwargs['width'] = -bar_kwargs['width']
             ax.bar(x_name, simulations_to_plot.data_to_plot['mean'],
-                   color='white', edgecolor=simu_colors, **bar_kwargs,
+                   color='white', edgecolor=color, **bar_kwargs,
                    label='simulation')
 
-    def generate_scatterplot(self, ax: 'matplotlib.pyplot.Axes',
-                             dataplot: DataPlot,
-                             plotTypeData: str) -> None:
+    def generate_scatterplot(
+            self,
+            ax: 'matplotlib.pyplot.Axes',
+            dataplot: DataPlot,
+            plotTypeData: str
+    ) -> None:
         """
         Generate scatterplot.
 
@@ -211,22 +226,23 @@ class MPLPlotter(Plotter):
         plotTypeData:
             Specifies how replicates should be handled.
         """
-
         measurements_to_plot, simulations_to_plot = \
             self.data_provider.get_data_to_plot(dataplot,
                                                 plotTypeData == PROVIDED)
 
-        if simulations_to_plot is None:
+        if simulations_to_plot is None or measurements_to_plot is None:
             raise NotImplementedError('Scatter plots do not work without'
                                       ' simulation data')
         ax.scatter(measurements_to_plot.data_to_plot['mean'],
                    simulations_to_plot.data_to_plot['mean'],
                    label=getattr(dataplot, LEGEND_ENTRY))
-        ax = self._square_plot_equal_ranges(ax)
+        self._square_plot_equal_ranges(ax)
 
-    def generate_subplot(self,
-                         ax,
-                         subplot: Subplot) -> None:
+    def generate_subplot(
+            self,
+            ax: plt.Axes,
+            subplot: Subplot
+    ) -> None:
         """
         Generate subplot based on markup provided by subplot.
 
@@ -237,14 +253,13 @@ class MPLPlotter(Plotter):
         subplot:
             Subplot visualization settings.
         """
-
         # set yScale
         if subplot.yScale == LIN:
             ax.set_yscale("linear")
         elif subplot.yScale == LOG10:
             ax.set_yscale("log")
         elif subplot.yScale == LOG:
-            ax.set_yscale("log", basey=np.e)
+            ax.set_yscale("log", base=np.e)
 
         if subplot.plotTypeSimulation == BAR_PLOT:
             for data_plot in subplot.data_plots:
@@ -266,23 +281,21 @@ class MPLPlotter(Plotter):
             for data_plot in subplot.data_plots:
                 self.generate_scatterplot(ax, data_plot, subplot.plotTypeData)
         else:
-
             # set xScale
             if subplot.xScale == LIN:
                 ax.set_xscale("linear")
             elif subplot.xScale == LOG10:
                 ax.set_xscale("log")
             elif subplot.xScale == LOG:
-                ax.set_xscale("log", basex=np.e)
+                ax.set_xscale("log", base=np.e)
             # equidistant
             elif subplot.xScale == 'order':
                 ax.set_xscale("linear")
                 # check if conditions are monotone decreasing or increasing
-                if np.all(
-                        np.diff(subplot.conditions) < 0):  # monot. decreasing
-                    xlabel = subplot.conditions[::-1]  # reversing
-                    conditions = range(len(subplot.conditions))[
-                                 ::-1]  # reversing
+                if np.all(np.diff(subplot.conditions) < 0):
+                    # monot. decreasing -> reverse
+                    xlabel = subplot.conditions[::-1]
+                    conditions = range(len(subplot.conditions))[::-1]
                     ax.set_xticks(range(len(conditions)), xlabel)
                 elif np.all(np.diff(subplot.conditions) > 0):
                     xlabel = subplot.conditions
@@ -305,10 +318,9 @@ class MPLPlotter(Plotter):
         if subplot.yScale == LOG:
             ax.yaxis.set_major_formatter(mtick.FuncFormatter(ticks))
 
-        if not subplot.plotTypeSimulation == BAR_PLOT:
+        if subplot.plotTypeSimulation != BAR_PLOT:
             ax.legend()
         ax.set_title(subplot.plotName)
-        ax.relim()
         if subplot.xlim:
             ax.set_xlim(subplot.xlim)
         if subplot.ylim:
@@ -316,13 +328,14 @@ class MPLPlotter(Plotter):
         ax.autoscale_view()
 
         # Beautify plots
-        ax.set_xlabel(
-            subplot.xLabel)
-        ax.set_ylabel(
-            subplot.yLabel)
+        ax.set_xlabel(subplot.xLabel)
+        ax.set_ylabel(subplot.yLabel)
 
-    def generate_figure(self, subplot_dir: Optional[str] = None
-                        ) -> Optional[Dict[str, plt.Subplot]]:
+    def generate_figure(
+            self,
+            subplot_dir: Optional[str] = None,
+            format_: str = 'png',
+    ) -> Optional[Dict[str, plt.Subplot]]:
         """
         Generate the full figure based on the markup in the figure attribute.
 
@@ -331,6 +344,9 @@ class MPLPlotter(Plotter):
         subplot_dir:
             A path to the folder where single subplots should be saved.
             PlotIDs will be taken as file names.
+        format_:
+            File format for the generated figure.
+            (See :py:func:`matplotlib.pyplot.savefig` for supported options).
 
         Returns
         -------
@@ -339,25 +355,14 @@ class MPLPlotter(Plotter):
         None:
             In case subplots are saved to file.
         """
-
-        # Set Options for plots
-        # possible options: see: plt.rcParams.keys()
-
-        # TODO: or some other style
-        plt.style.use('tableau-colorblind10')
-
-        plt.rcParams['font.size'] = 12
-        plt.rcParams['axes.titlesize'] = 12
-        plt.rcParams['figure.figsize'] = self.figure.size
-        plt.rcParams['errorbar.capsize'] = 2
-
         if subplot_dir is None:
             # compute, how many rows and columns we need for the subplots
             num_row = int(np.round(np.sqrt(self.figure.num_subplots)))
             num_col = int(np.ceil(self.figure.num_subplots / num_row))
 
-            fig, axes = plt.subplots(num_row, num_col, squeeze=False)
-            fig.set_tight_layout(True)
+            fig, axes = plt.subplots(num_row, num_col, squeeze=False,
+                                     figsize=self.figure.size)
+            fig.set_layout_engine("tight")
 
             for ax in axes.flat[self.figure.num_subplots:]:
                 ax.remove()
@@ -365,20 +370,24 @@ class MPLPlotter(Plotter):
             axes = dict(zip([plot.plotId for plot in self.figure.subplots],
                             axes.flat))
 
-        for idx, subplot in enumerate(self.figure.subplots):
+        for subplot in self.figure.subplots:
             if subplot_dir is not None:
-                fig, ax = plt.subplots()
-                fig.set_tight_layout(True)
+                fig, ax = plt.subplots(figsize=self.figure.size)
+                fig.set_layout_engine("tight")
             else:
                 ax = axes[subplot.plotId]
 
-            self.generate_subplot(ax, subplot)
+            try:
+                self.generate_subplot(ax, subplot)
+            except Exception as e:
+                raise RuntimeError(
+                    f"Error plotting {getattr(subplot, PLOT_ID)}.") from e
 
             if subplot_dir is not None:
                 # TODO: why this doesn't work?
                 plt.tight_layout()
                 plt.savefig(os.path.join(subplot_dir,
-                                         f'{subplot.plotId}.png'))
+                                         f'{subplot.plotId}.{format_}'))
                 plt.close()
 
         if subplot_dir is None:
@@ -423,6 +432,8 @@ class SeabornPlotter(Plotter):
     def __init__(self, figure: Figure, data_provider: DataProvider):
         super().__init__(figure, data_provider)
 
-    def generate_figure(self, subplot_dir: Optional[str] = None
-                        ) -> Optional[Dict[str, plt.Subplot]]:
+    def generate_figure(
+            self,
+            subplot_dir: Optional[str] = None
+    ) -> Optional[Dict[str, plt.Subplot]]:
         pass
