@@ -1,16 +1,17 @@
 """PEtab visualization data selection and visualization settings classes"""
+import warnings
+from numbers import Number, Real
+from pathlib import Path
+from typing import Dict, List, Optional, Tuple, Union, Literal
+
 import numpy as np
 import pandas as pd
 
-from typing import Dict, List, Optional, Tuple, Union
-
-from .helper_functions import (generate_dataset_id_col,
-                               create_dataset_id_list_new)
-from .. import measurements, core, conditions
-from ..problem import Problem
+from .helper_functions import (create_dataset_id_list_new,
+                               generate_dataset_id_col)
+from .. import conditions, core, measurements
 from ..C import *
-from numbers import Number, Real
-import warnings
+from ..problem import Problem
 
 __all__ = ['DataSeries', 'DataPlot', 'Subplot', 'Figure', 'DataProvider',
            'VisSpecParser']
@@ -72,10 +73,8 @@ class DataSeries:
             self.conditions += offset
 
     def add_y_offset(self, offset):
-        self.data_to_plot['mean'] = \
-            self.data_to_plot['mean'] + offset
-        self.data_to_plot['repl'] = \
-            self.data_to_plot['repl'] + offset
+        self.data_to_plot['mean'] += offset
+        self.data_to_plot['repl'] += offset
 
     def add_offsets(self, x_offset=0, y_offset=0) -> None:
         """
@@ -131,6 +130,9 @@ class DataPlot:
 
         return cls(vis_spec_dict)
 
+    def __repr__(self):
+        return f"{self.__class__.__name__}({self.__dict__})"
+
 
 class Subplot:
     """
@@ -348,9 +350,9 @@ class Figure:
                                  dataplot.__dict__ if key in
                                  VISUALIZATION_DF_SINGLE_PLOT_LEVEL_COLS}
                 row = {**subplot_level, **dataset_level}
-                for key in row:
+                for key, value in row.items():
                     if key in visu_dict:
-                        visu_dict[key].append(row[key])
+                        visu_dict[key].append(value)
                     else:
                         visu_dict[key] = [row[key]]
         visu_df = pd.DataFrame.from_dict(visu_dict)
@@ -481,9 +483,13 @@ class DataProvider:
 
         return uni_condition_id, col_name_unique, conditions_
 
-    def get_data_series(self, data_df: pd.DataFrame, data_col: str,
-                        dataplot: DataPlot,
-                        provided_noise: bool) -> DataSeries:
+    def get_data_series(
+            self,
+            data_df: pd.DataFrame,
+            data_col: Literal['measurement', 'simulation'],
+            dataplot: DataPlot,
+            provided_noise: bool
+    ) -> DataSeries:
         """
         Get data to plot from measurement or simulation DataFrame.
 
@@ -500,10 +506,8 @@ class DataProvider:
         -------
         Data to plot
         """
-
         uni_condition_id, col_name_unique, conditions_ = \
-            self._get_independent_var_values(data_df,
-                                             dataplot)
+            self._get_independent_var_values(data_df, dataplot)
 
         dataset_id = getattr(dataplot, DATASET_ID)
 
@@ -596,7 +600,6 @@ class DataProvider:
                                                        SIMULATION,
                                                        dataplot,
                                                        provided_noise)
-
         return measurements_to_plot, simulations_to_plot
 
 
@@ -609,19 +612,20 @@ class VisSpecParser:
     Figure instance, a DataProvider instance is created that will be
     responsible for the data selection and manipulation.
     """
-    def __init__(self,
-                 conditions_data: Union[str, pd.DataFrame],
-                 exp_data: Optional[Union[str, pd.DataFrame]] = None,
-                 sim_data: Optional[Union[str, pd.DataFrame]] = None,
-                 ):
-        if isinstance(conditions_data, str):
+    def __init__(
+            self,
+            conditions_data: Union[str, Path, pd.DataFrame],
+            exp_data: Optional[Union[str, Path, pd.DataFrame]] = None,
+            sim_data: Optional[Union[str, Path, pd.DataFrame]] = None,
+    ):
+        if isinstance(conditions_data, (str, Path)):
             conditions_data = conditions.get_condition_df(conditions_data)
 
         # import from file in case experimental data is provided in file
-        if isinstance(exp_data, str):
+        if isinstance(exp_data, (str, Path)):
             exp_data = measurements.get_measurement_df(exp_data)
 
-        if isinstance(sim_data, str):
+        if isinstance(sim_data, (str, Path)):
             sim_data = core.get_simulation_df(sim_data)
 
         if exp_data is None and sim_data is None:
@@ -644,8 +648,10 @@ class VisSpecParser:
             None else self.simulations_data
 
     @staticmethod
-    def create_subplot(plot_id: str,
-                       subplot_vis_spec: pd.DataFrame) -> Subplot:
+    def create_subplot(
+            plot_id: str,
+            subplot_vis_spec: pd.DataFrame
+    ) -> Subplot:
         """
         Create subplot.
 
@@ -662,7 +668,6 @@ class VisSpecParser:
 
                 Subplot
         """
-
         subplot_columns = [col for col in subplot_vis_spec.columns if col in
                            VISUALIZATION_DF_SUBPLOT_LEVEL_COLS]
         subplot = Subplot.from_df(plot_id,
@@ -678,9 +683,10 @@ class VisSpecParser:
 
         return subplot
 
-    def parse_from_vis_spec(self,
-                            vis_spec: Optional[Union[str, pd.DataFrame]],
-                            ) -> Tuple[Figure, DataProvider]:
+    def parse_from_vis_spec(
+            self,
+            vis_spec: Optional[Union[str, Path, pd.DataFrame]],
+    ) -> Tuple[Figure, DataProvider]:
         """
         Get visualization settings from a visualization specification.
 
@@ -695,9 +701,8 @@ class VisSpecParser:
 
         A figure template with visualization settings and a data provider
         """
-
         # import visualization specification, if file was specified
-        if isinstance(vis_spec, str):
+        if isinstance(vis_spec, (str, Path)):
             vis_spec = core.get_visualization_df(vis_spec)
 
         if DATASET_ID not in vis_spec.columns:
@@ -874,11 +879,12 @@ class VisSpecParser:
         plot_id_column = ['plot%s' % str(ind + 1) for ind, inner_list in
                           enumerate(dataset_id_list) for _ in inner_list]
 
-        columns_dict = {PLOT_ID: plot_id_column,
-                        DATASET_ID: dataset_id_column,
-                        LEGEND_ENTRY: dataset_label_column,
-                        Y_VALUES: yvalues_column}
-        return columns_dict
+        return {
+            PLOT_ID: plot_id_column,
+            DATASET_ID: dataset_id_column,
+            LEGEND_ENTRY: dataset_label_column,
+            Y_VALUES: yvalues_column
+        }
 
     def _create_legend(self, dataset_id: str) -> str:
         """
@@ -905,7 +911,7 @@ class VisSpecParser:
             cond_name = cond_id
         else:
             cond_name = tmp[CONDITION_NAME]
-        return cond_name + ' - ' + obs_id
+        return f'{cond_name} - {obs_id}'
 
     def _expand_vis_spec_settings(self, vis_spec: pd.DataFrame):
         """
@@ -925,34 +931,32 @@ class VisSpecParser:
         if DATASET_ID in vis_spec.columns:
             raise ValueError(f"visualization specification expansion is "
                              f"unnecessary if column {DATASET_ID} is present")
-        else:
-            vis_spec_exp = pd.DataFrame()
-            if vis_spec.empty:
-                # in case of empty spec all measurements corresponding to each
-                # observable will be plotted on a separate subplot
+
+        if vis_spec.empty:
+            # in case of empty spec all measurements corresponding to each
+            # observable will be plotted on a separate subplot
+            observable_ids = self._data_df[OBSERVABLE_ID].unique()
+
+            vis_spec_exp_rows = [
+                self._vis_spec_rows_for_obs(obs_id, {PLOT_ID: f'plot{idx}'})
+                for idx, obs_id in enumerate(observable_ids)
+            ]
+            return pd.concat(vis_spec_exp_rows, ignore_index=True)
+
+        vis_spec_exp_rows = []
+        for _, row in vis_spec.iterrows():
+            if Y_VALUES in row:
+                vis_spec_exp_rows.append(
+                    self._vis_spec_rows_for_obs(row[Y_VALUES], row.to_dict())
+                )
+            else:
                 observable_ids = self._data_df[OBSERVABLE_ID].unique()
 
-                for idx, obs_id in enumerate(observable_ids):
-                    obs_vis_spec = self._vis_spec_rows_for_obs(
-                        obs_id, {PLOT_ID: f'plot{idx}'})
-                    vis_spec_exp = vis_spec_exp.append(obs_vis_spec,
-                                                       ignore_index=True)
-            else:
-                for _, row in vis_spec.iterrows():
-                    if Y_VALUES in row:
-                        obs_vis_spec = self._vis_spec_rows_for_obs(
-                            row[Y_VALUES], row.to_dict())
-                        vis_spec_exp = vis_spec_exp.append(obs_vis_spec,
-                                                           ignore_index=True)
-                    else:
-                        observable_ids = self._data_df[OBSERVABLE_ID].unique()
-
-                        for obs_id in observable_ids:
-                            obs_vis_spec = self._vis_spec_rows_for_obs(
-                                obs_id, row.to_dict())
-                            vis_spec_exp = vis_spec_exp.append(
-                                obs_vis_spec, ignore_index=True)
-        return vis_spec_exp
+                for obs_id in observable_ids:
+                    vis_spec_exp_rows.append(
+                        self._vis_spec_rows_for_obs(obs_id, row.to_dict())
+                    )
+        return pd.concat(vis_spec_exp_rows, ignore_index=True)
 
     def _vis_spec_rows_for_obs(self, obs_id: str, settings: dict
                                ) -> pd.DataFrame:
