@@ -1,8 +1,8 @@
 """Code regarding the PEtab YAML config files"""
 
 import os
-from pathlib import Path as _Path
-from typing import Any, Dict, Union, Optional, List
+from pathlib import Path
+from typing import Any, Dict, List, Optional, Union
 
 import jsonschema
 import numpy as np
@@ -11,15 +11,25 @@ from pandas.io.common import get_handle
 
 from .C import *  # noqa: F403
 
-SCHEMA = os.path.join(os.path.abspath(os.path.dirname(__file__)),
-                      "petab_schema.yaml")
+# directory with PEtab yaml schema files
+SCHEMA_DIR = Path(__file__).parent / "schemas"
+# map of version number to validation schema
+SCHEMAS = {
+    '1': SCHEMA_DIR / "petab_schema.v1.0.0.yaml",
+    '1.0.0': SCHEMA_DIR / "petab_schema.v1.0.0.yaml",
+    '2.0.0': SCHEMA_DIR / "petab_schema.v2.0.0.yaml",
+}
+
 __all__ = ['validate', 'validate_yaml_syntax', 'validate_yaml_semantics',
            'load_yaml', 'is_composite_problem',
            'assert_single_condition_and_sbml_file', 'write_yaml',
            'create_problem_yaml']
 
 
-def validate(yaml_config: Union[Dict, str], path_prefix: Optional[str] = None):
+def validate(
+        yaml_config: Union[Dict, str, Path],
+        path_prefix: Union[None, str, Path] = None,
+):
     """Validate syntax and semantics of PEtab config YAML
 
     Arguments:
@@ -37,7 +47,7 @@ def validate(yaml_config: Union[Dict, str], path_prefix: Optional[str] = None):
 
 
 def validate_yaml_syntax(
-        yaml_config: Union[Dict, str],
+        yaml_config: Union[Dict, str, Path],
         schema: Union[None, Dict, str] = None):
     """Validate PEtab YAML file syntax
 
@@ -48,20 +58,29 @@ def validate_yaml_syntax(
             Custom schema for validation
 
     Raises:
-        see jsonschema.validate
+        see :func:`jsonschema.validate`
     """
-
-    if schema is None:
-        schema = SCHEMA
-
-    schema = load_yaml(schema)
     yaml_config = load_yaml(yaml_config)
 
+    if schema is None:
+        # try get PEtab version from yaml file
+        #  if this is not the available, the file is not valid anyways,
+        #  but let's still use the latest PEtab schema for full validation
+        version = yaml_config.get(FORMAT_VERSION, None) \
+                      or list(SCHEMAS.values())[-1]
+        try:
+            schema = SCHEMAS[str(version)]
+        except KeyError as e:
+            raise ValueError("Unknown PEtab version given in problem "
+                             f"specification: {version}") from e
+    schema = load_yaml(schema)
     jsonschema.validate(instance=yaml_config, schema=schema)
 
 
 def validate_yaml_semantics(
-        yaml_config: Union[Dict, str], path_prefix: Optional[str] = None):
+        yaml_config: Union[Dict, str, Path],
+        path_prefix: Union[None, str, Path] = None
+):
     """Validate PEtab YAML file semantics
 
     Check for existence of files. Assumes valid syntax.
@@ -80,8 +99,8 @@ def validate_yaml_semantics(
         AssertionError: in case of problems
     """
     if not path_prefix:
-        if isinstance(yaml_config, str):
-            path_prefix = os.path.dirname(yaml_config)
+        if isinstance(yaml_config, (str, Path)):
+            path_prefix = os.path.dirname(str(yaml_config))
         else:
             path_prefix = ""
 
@@ -109,7 +128,7 @@ def validate_yaml_semantics(
                     _check_file(os.path.join(path_prefix, filename), field)
 
 
-def load_yaml(yaml_config: Union[Dict, str]) -> Dict:
+def load_yaml(yaml_config: Union[Dict, Path, str]) -> Dict:
     """Load YAML
 
     Convenience function to allow for providing YAML inputs as filename, URL
@@ -128,11 +147,12 @@ def load_yaml(yaml_config: Union[Dict, str]) -> Dict:
     if isinstance(yaml_config, dict):
         return yaml_config
 
-    handle = get_handle(yaml_config, mode='r').handle
-    return yaml.safe_load(handle)
+    with get_handle(yaml_config, mode='r') as io_handle:
+        data = yaml.safe_load(io_handle.handle)
+    return data
 
 
-def is_composite_problem(yaml_config: Union[Dict, str]) -> bool:
+def is_composite_problem(yaml_config: Union[Dict, str, Path]) -> bool:
     """Does this YAML file comprise multiple models?
 
     Arguments:
@@ -164,31 +184,33 @@ def assert_single_condition_and_sbml_file(problem_config: Dict) -> None:
             'implemented.')
 
 
-def write_yaml(yaml_config: Dict[str, Any], filename: str) -> None:
+def write_yaml(
+        yaml_config: Dict[str, Any],
+        filename: Union[str, Path]
+) -> None:
     """Write PEtab YAML file
 
     Arguments:
         yaml_config: Data to write
         filename: File to create
     """
-
     with open(filename, 'w') as outfile:
         yaml.dump(yaml_config, outfile, default_flow_style=False,
                   sort_keys=False)
 
 
 def create_problem_yaml(
-        sbml_files: Union[str, List[str]],
-        condition_files: Union[str, List[str]],
-        measurement_files: Union[str, List[str]],
-        parameter_file: str,
-        observable_files: Union[str, List[str]],
-        yaml_file: str,
-        visualization_files: Optional[Union[str, List[str]]] = None,
+        sbml_files: Union[str, Path, List[Union[str, Path]]],
+        condition_files: Union[str, Path, List[Union[str, Path]]],
+        measurement_files: Union[str, Path, List[Union[str, Path]]],
+        parameter_file: Union[str, Path],
+        observable_files: Union[str, Path, List[Union[str, Path]]],
+        yaml_file: Union[str, Path],
+        visualization_files:
+        Optional[Union[str, Path, List[Union[str, Path]]]] = None,
         relative_paths: bool = True,
 ) -> None:
-    """
-    Create and write default YAML file for a single PEtab problem
+    """Create and write default YAML file for a single PEtab problem
 
     Arguments:
         sbml_files: Path of SBML model file or list of such
@@ -204,19 +226,19 @@ def create_problem_yaml(
             location of the YAML file. If ``False``, then paths are left
             unchanged.
     """
-    if isinstance(sbml_files, str):
+    if isinstance(sbml_files, (Path, str)):
         sbml_files = [sbml_files]
-    if isinstance(condition_files, str):
+    if isinstance(condition_files, (Path, str)):
         condition_files = [condition_files]
-    if isinstance(measurement_files, str):
+    if isinstance(measurement_files, (Path, str)):
         measurement_files = [measurement_files]
-    if isinstance(observable_files, str):
+    if isinstance(observable_files, (Path, str)):
         observable_files = [observable_files]
-    if isinstance(visualization_files, str):
+    if isinstance(visualization_files, (Path, str)):
         visualization_files = [visualization_files]
 
     if relative_paths:
-        yaml_file_dir = _Path(yaml_file).parent
+        yaml_file_dir = Path(yaml_file).parent
 
         def get_rel_to_yaml(paths: Union[List[str], None]):
             if paths is None:
@@ -239,7 +261,7 @@ def create_problem_yaml(
                    SBML_FILES: sbml_files,
                    OBSERVABLE_FILES: observable_files}
     if visualization_files is not None:
-        problem_dic.update({'visualization_files': visualization_files})
+        problem_dic.update({VISUALIZATION_FILES: visualization_files})
     yaml_dic = {PARAMETER_FILE: parameter_file,
                 FORMAT_VERSION: 1,
                 PROBLEMS: [problem_dic]}
